__all__ = [
    "apply_manual_api_refinement"
]

from typing import cast, Sequence, Callable, Iterable, Optional

from .nodes import (NamespaceNode, FunctionNode, OptionalTypeNode, TypeNode,
                    ClassProperty, PrimitiveTypeNode, ASTNodeTypeNode,
                    AggregatedTypeNode, CallableTypeNode, AnyTypeNode,
                    TupleTypeNode, UnionTypeNode, ProtocolClassNode,
                    DictTypeNode, ClassTypeNode, AliasRefTypeNode)
from .ast_utils import (find_function_node, SymbolName,
                        for_each_function_overload)
from .types_conversion import create_type_node


def apply_manual_api_refinement(root: NamespaceNode) -> None:
    refine_highgui_module(root)
    refine_cuda_module(root)
    export_matrix_type_constants(root)
    refine_dnn_module(root)
    # Export OpenCV exception class
    builtin_exception = root.add_class("Exception")
    builtin_exception.is_exported = False
    root.add_class("error", (builtin_exception, ), ERROR_CLASS_PROPERTIES)
    for symbol_name, refine_symbol in NODES_TO_REFINE.items():
        refine_symbol(root, symbol_name)
    version_constant = root.add_constant("__version__", "<unused>")
    version_constant._value_type = "str"

    convert_returned_scalar_to_tuple(root)

    """
    def redirectError(
        onError: Callable[[int, str, str, str, int], None] | None
    ) -> None: ...
    """
    root.add_function("redirectError", [
        FunctionNode.Arg(
            "onError",
            OptionalTypeNode(
                CallableTypeNode(
                    "ErrorCallback",
                    [
                        PrimitiveTypeNode.int_(),
                        PrimitiveTypeNode.str_(),
                        PrimitiveTypeNode.str_(),
                        PrimitiveTypeNode.str_(),
                        PrimitiveTypeNode.int_()
                    ]
                )
            )
        )
    ])


def make_optional_none_return(root_node: NamespaceNode,
                              function_symbol_name: SymbolName) -> None:
    """
    Make return type Optional[MatLike],
    for the functions that may return None.
    """
    function = find_function_node(root_node, function_symbol_name)
    for overload in function.overloads:
        if overload.return_type is not None:
            if not isinstance(overload.return_type.type_node, OptionalTypeNode):
                overload.return_type.type_node = OptionalTypeNode(
                    overload.return_type.type_node
                )

def export_matrix_type_constants(root: NamespaceNode) -> None:
    MAX_PREDEFINED_CHANNELS = 4

    depth_names = ("CV_8U", "CV_8S", "CV_16U", "CV_16S", "CV_32U", "CV_32S",
                   "CV_64U", "CV_64S", "CV_32F", "CV_64F", "CV_16F", "CV_16BF" "CV_Bool")
    for depth_value, depth_name in enumerate(depth_names):
        # Export depth constants
        root.add_constant(depth_name, str(depth_value))
        # Export predefined types
        for c in range(MAX_PREDEFINED_CHANNELS):
            root.add_constant(f"{depth_name}C{c + 1}",
                              f"{depth_value + 8 * c}")
        # Export type creation function
        root.add_function(
            f"{depth_name}C",
            (FunctionNode.Arg("channels", PrimitiveTypeNode.int_()), ),
            FunctionNode.RetType(PrimitiveTypeNode.int_())
        )
    # Export CV_MAKETYPE
    root.add_function(
        "CV_MAKETYPE",
        (FunctionNode.Arg("depth", PrimitiveTypeNode.int_()),
         FunctionNode.Arg("channels", PrimitiveTypeNode.int_())),
        FunctionNode.RetType(PrimitiveTypeNode.int_())
    )


def make_optional_arg(*arg_names: str) -> Callable[[NamespaceNode, SymbolName], None]:
    def _make_optional_arg(root_node: NamespaceNode,
                           function_symbol_name: SymbolName) -> None:
        function = find_function_node(root_node, function_symbol_name)
        for arg_name in arg_names:
            found_overload_with_arg = False

            for overload in function.overloads:
                arg_idx = _find_argument_index(overload.arguments, arg_name)

                # skip overloads without this argument
                if arg_idx is None:
                    continue

                # Avoid multiplying optional qualification
                if isinstance(overload.arguments[arg_idx].type_node, OptionalTypeNode):
                    continue

                overload.arguments[arg_idx].type_node = OptionalTypeNode(
                    cast(TypeNode, overload.arguments[arg_idx].type_node)
                )

                found_overload_with_arg = True

            if not found_overload_with_arg:
                raise RuntimeError(
                    f"Failed to find argument with name: '{arg_name}'"
                    f" in '{function_symbol_name.name}' overloads"
                )

    return _make_optional_arg


def convert_returned_scalar_to_tuple(root: NamespaceNode) -> None:
    """Force `tuple[float, float, float, float]` usage instead of Scalar alias
    for return types due to `pyopencv_from` specialization for Scalar type.
    """

    float_4_tuple_node = TupleTypeNode(
        "ScalarOutput",
        items=(PrimitiveTypeNode.float_(),) * 4
    )

    def fix_scalar_return_type(fn: FunctionNode.Overload):
        if fn.return_type is None:
            return
        if fn.return_type.type_node.typename == "Scalar":
            fn.return_type.type_node = float_4_tuple_node

    for overload in for_each_function_overload(root):
        fix_scalar_return_type(overload)

    for ns in root.namespaces.values():
        for overload in for_each_function_overload(ns):
            fix_scalar_return_type(overload)


def refine_cuda_module(root: NamespaceNode) -> None:
    def fix_cudaoptflow_enums_names() -> None:
        for class_name in ("NvidiaOpticalFlow_1_0", "NvidiaOpticalFlow_2_0"):
            if class_name not in cuda_root.classes:
                continue
            opt_flow_class = cuda_root.classes[class_name]
            _trim_class_name_from_argument_types(
                for_each_function_overload(opt_flow_class), class_name
            )

    def fix_namespace_usage_scope(cuda_ns: NamespaceNode) -> None:
        USED_TYPES = ("GpuMat", "Stream")

        def fix_type_usage(type_node: TypeNode) -> None:
            if isinstance(type_node, AggregatedTypeNode):
                for item in type_node.items:
                    fix_type_usage(item)
            if isinstance(type_node, ASTNodeTypeNode):
                if type_node._typename in USED_TYPES:
                    type_node._typename = f"cuda_{type_node._typename}"

        for overload in for_each_function_overload(cuda_ns):
            if overload.return_type is not None:
                fix_type_usage(overload.return_type.type_node)
            for type_node in [arg.type_node for arg in overload.arguments
                              if arg.type_node is not None]:
                fix_type_usage(type_node)

    if "cuda" not in root.namespaces:
        return
    cuda_root = root.namespaces["cuda"]
    fix_cudaoptflow_enums_names()
    for ns in [ns for ns_name, ns in root.namespaces.items()
               if ns_name.startswith("cuda")]:
        fix_namespace_usage_scope(ns)


def refine_highgui_module(root: NamespaceNode) -> None:
    # Check if library is built with enabled highgui module
    if "destroyAllWindows" not in root.functions:
        return
    """
    def createTrackbar(trackbarName: str,
                       windowName: str,
                       value: int,
                       count: int,
                       onChange: Callable[[int], None]) -> None: ...
    """
    root.add_function(
        "createTrackbar",
        [
            FunctionNode.Arg("trackbarName", PrimitiveTypeNode.str_()),
            FunctionNode.Arg("windowName", PrimitiveTypeNode.str_()),
            FunctionNode.Arg("value", PrimitiveTypeNode.int_()),
            FunctionNode.Arg("count", PrimitiveTypeNode.int_()),
            FunctionNode.Arg("onChange",
                             CallableTypeNode("TrackbarCallback",
                                              PrimitiveTypeNode.int_("int"))),
        ]
    )
    """
    def createButton(buttonName: str,
                     onChange: Callable[[tuple[int] | tuple[int, Any]], None],
                     userData: Any | None = ...,
                     buttonType: int = ...,
                     initialButtonState: int = ...) -> None: ...
    """
    root.add_function(
        "createButton",
        [
            FunctionNode.Arg("buttonName", PrimitiveTypeNode.str_()),
            FunctionNode.Arg(
                "onChange",
                CallableTypeNode(
                    "ButtonCallback",
                    UnionTypeNode(
                        "onButtonChangeCallbackData",
                        [
                            TupleTypeNode("onButtonChangeCallbackData",
                                          [PrimitiveTypeNode.int_(), ]),
                            TupleTypeNode("onButtonChangeCallbackData",
                                          [PrimitiveTypeNode.int_(),
                                           AnyTypeNode("void*")])
                        ]
                    )
                )),
            FunctionNode.Arg("userData",
                             OptionalTypeNode(AnyTypeNode("void*")),
                             default_value="None"),
            FunctionNode.Arg("buttonType", PrimitiveTypeNode.int_(),
                             default_value="0"),
            FunctionNode.Arg("initialButtonState", PrimitiveTypeNode.int_(),
                             default_value="0")
        ]
    )
    """
    def setMouseCallback(
        windowName: str,
        onMouse: Callback[[int, int, int, int, Any | None], None],
        param: Any | None = ...
    ) -> None: ...
    """
    root.add_function(
        "setMouseCallback",
        [
            FunctionNode.Arg("windowName", PrimitiveTypeNode.str_()),
            FunctionNode.Arg(
                "onMouse",
                CallableTypeNode("MouseCallback", [
                    PrimitiveTypeNode.int_(),
                    PrimitiveTypeNode.int_(),
                    PrimitiveTypeNode.int_(),
                    PrimitiveTypeNode.int_(),
                    OptionalTypeNode(AnyTypeNode("void*"))
                ])
            ),
            FunctionNode.Arg("param", OptionalTypeNode(AnyTypeNode("void*")),
                             default_value="None")
        ]
    )


def refine_dnn_module(root: NamespaceNode) -> None:
    if "dnn" not in root.namespaces:
        return
    dnn_module = root.namespaces["dnn"]

    """
    class LayerProtocol(Protocol):
        def __init__(
            self, params: dict[str, DictValue],
            blobs: typing.Sequence[cv2.typing.MatLike]
        ) -> None: ...

        def getMemoryShapes(
            self, inputs: typing.Sequence[typing.Sequence[int]]
        ) -> typing.Sequence[typing.Sequence[int]]: ...

        def forward(
            self, inputs: typing.Sequence[cv2.typing.MatLike]
        ) -> typing.Sequence[cv2.typing.MatLike]: ...
    """
    layer_proto = ProtocolClassNode("LayerProtocol", dnn_module)
    layer_proto.add_function(
        "__init__",
        arguments=[
            FunctionNode.Arg(
                "params",
                DictTypeNode(
                    "LayerParams", PrimitiveTypeNode.str_(),
                    create_type_node("cv::dnn::DictValue")
                )
            ),
            FunctionNode.Arg("blobs", create_type_node("vector<cv::Mat>"))
        ]
    )
    layer_proto.add_function(
        "getMemoryShapes",
        arguments=[
            FunctionNode.Arg("inputs",
                             create_type_node("vector<vector<int>>"))
        ],
        return_type=FunctionNode.RetType(
            create_type_node("vector<vector<int>>")
        )
    )
    layer_proto.add_function(
        "forward",
        arguments=[
            FunctionNode.Arg("inputs", create_type_node("vector<cv::Mat>"))
        ],
        return_type=FunctionNode.RetType(create_type_node("vector<cv::Mat>"))
    )

    """
    def dnn_registerLayer(layerTypeName: str,
                          layerClass: typing.Type[LayerProtocol]) -> None: ...
    """
    root.add_function(
        "dnn_registerLayer",
        arguments=[
            FunctionNode.Arg("layerTypeName", PrimitiveTypeNode.str_()),
            FunctionNode.Arg(
                "layerClass",
                ClassTypeNode(ASTNodeTypeNode(
                    layer_proto.export_name, f"dnn.{layer_proto.export_name}"
                ))
            )
        ]
    )

    """
    def dnn_unregisterLayer(layerTypeName: str) -> None: ...
    """
    root.add_function(
        "dnn_unregisterLayer",
        arguments=[
            FunctionNode.Arg("layerTypeName", PrimitiveTypeNode.str_())
        ]
    )


def _trim_class_name_from_argument_types(
    overloads: Iterable[FunctionNode.Overload],
    class_name: str
) -> None:
    separator = f"{class_name}_"
    for overload in overloads:
        for arg in [arg for arg in overload.arguments
                    if arg.type_node is not None]:
            ast_node = cast(ASTNodeTypeNode, arg.type_node)
            if class_name in ast_node.ctype_name:
                fixed_name = ast_node._typename.split(separator)[-1]
                ast_node._typename = fixed_name


def _find_argument_index(arguments: Sequence[FunctionNode.Arg],
                         name: str) -> Optional[int]:
    for i, arg in enumerate(arguments):
        if arg.name == name:
            return i
    return None


def make_matlike_or_scalar_arg(*arg_names: str) -> Callable[[NamespaceNode, SymbolName], None]:
    """Make arguments accept both MatLike and Scalar types.

    This is used for functions like inRange where the C++ InputArray parameter
    can accept both Mat objects and Scalar values (tuples, floats, etc.).

    Example: cv2.inRange(img, (0, 0, 0), (255, 255, 255)) should be valid.
    """
    def _make_matlike_or_scalar_arg(root_node: NamespaceNode,
                                     function_symbol_name: SymbolName) -> None:
        from .predefined_types import PREDEFINED_TYPES

        function = find_function_node(root_node, function_symbol_name)
        for arg_name in arg_names:
            found_overload_with_arg = False

            for overload in function.overloads:
                arg_idx = _find_argument_index(overload.arguments, arg_name)

                # skip overloads without this argument
                if arg_idx is None:
                    continue

                current_type = overload.arguments[arg_idx].type_node

                # Check if it's already a union or if it already includes Scalar
                if isinstance(current_type, UnionTypeNode):
                    # Check if Scalar is already in the union
                    has_scalar = any(
                        isinstance(item, AliasRefTypeNode) and item.typename == "Scalar"
                        for item in current_type.items
                    )
                    if has_scalar:
                        continue
                    # Add Scalar to existing union
                    scalar_ref = AliasRefTypeNode("Scalar")
                    current_type.items = current_type.items + (scalar_ref,)
                else:
                    # Create a union of current type and Scalar
                    scalar_ref = AliasRefTypeNode("Scalar")
                    overload.arguments[arg_idx].type_node = UnionTypeNode(
                        f"{arg_name}_type",
                        (cast(TypeNode, current_type), scalar_ref)
                    )

                found_overload_with_arg = True

            if not found_overload_with_arg:
                raise RuntimeError(
                    f"Failed to find argument with name: '{arg_name}'"
                    f" in '{function_symbol_name.name}' overloads"
                )

    return _make_matlike_or_scalar_arg


NODES_TO_REFINE = {
    SymbolName(("cv", ), (), "resize"): make_optional_arg("dsize"),
    SymbolName(("cv", ), (), "calcHist"): make_optional_arg("mask"),
    SymbolName(("cv", ), (), "floodFill"): make_optional_arg("mask"),
    SymbolName(("cv", ), ("Feature2D", ), "detectAndCompute"): make_optional_arg("mask"),
    SymbolName(("cv", ), (), "findEssentialMat"): make_optional_arg(
        "distCoeffs1", "distCoeffs2", "dist_coeff1", "dist_coeff2"
    ),
    SymbolName(("cv", ), (), "drawFrameAxes"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "getOptimalNewCameraMatrix"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "initInverseRectificationMap"): make_optional_arg("distCoeffs", "R"),
    SymbolName(("cv", ), (), "initUndistortRectifyMap"): make_optional_arg("distCoeffs", "R"),
    SymbolName(("cv", ), (), "projectPoints"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "solveP3P"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "solvePnP"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "solvePnPGeneric"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "solvePnPRansac"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "solvePnPRefineLM"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "solvePnPRefineVVS"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "undistort"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "undistortPoints"): make_optional_arg("distCoeffs"),
    SymbolName(("cv", ), (), "calibrateCamera"): make_optional_arg("cameraMatrix", "distCoeffs"),
    SymbolName(("cv", "fisheye"), (), "initUndistortRectifyMap"): make_optional_arg("D"),
    SymbolName(("cv", ), (), "imread"): make_optional_none_return,
    SymbolName(("cv", ), (), "imdecode"): make_optional_none_return,
    SymbolName(("cv", ), (), "HoughCircles"): make_optional_none_return,
    SymbolName(("cv", ), (), "HoughLines"): make_optional_none_return,
    SymbolName(("cv", ), (), "HoughLinesP"): make_optional_none_return,
    # Fix for issue #28534: inRange should accept Scalar for lowerb and upperb
    SymbolName(("cv", ), (), "inRange"): make_matlike_or_scalar_arg("lowerb", "upperb"),
}

ERROR_CLASS_PROPERTIES = (
    ClassProperty("code", PrimitiveTypeNode.int_(), False),
    ClassProperty("err", PrimitiveTypeNode.str_(), False),
    ClassProperty("file", PrimitiveTypeNode.str_(), False),
    ClassProperty("func", PrimitiveTypeNode.str_(), False),
    ClassProperty("line", PrimitiveTypeNode.int_(), False),
    ClassProperty("msg", PrimitiveTypeNode.str_(), False),
)



# This file is part of OpenCV project.
# It is subject to the license terms in the LICENSE file found in the top-level directory
# of this distribution and at http://opencv.org/license.html.

"""
3d calibration visualisation tool.

Loads YAMLS from calibration.cpp sample code and displays an interactive 3d plot using meshlab and matplotlib

usage:
    python3 visualize_mono_calibration.py cam1.yml
    python3 visualize_mono_calibration.py cam1.yml cam2.yml --view board --export scene

Arguments:
    calib_files       YAML files with calibration data
    --view            Reference frame: {camera, board} (default:board)
    --export NAME     Export scene as name.obj/.mtl
    --no-gui          Disable GUI (only export)


"""

import argparse
import sys
from typing import Iterable, List, Tuple, Dict, Optional

import numpy as np
import cv2
import matplotlib.pyplot as plt
from matplotlib.widgets import SpanSelector
from mpl_toolkits.mplot3d.art3d import Poly3DCollection


def load_calib(
    filename: str,
) -> Tuple[List[float], np.ndarray, np.ndarray, np.ndarray, Tuple[int, int]]:
    fs = cv2.FileStorage(filename, cv2.FILE_STORAGE_READ)
    if not fs.isOpened():
        raise IOError(f"Could not open calibration file: {filename}")
    errs = (
        fs.getNode("per_view_reprojection_errors")
        .mat()
        .flatten()
        .astype(float)
        .tolist()
    )
    extr = fs.getNode("extrinsic_parameters").mat().astype(np.float64)
    # grid points may be stored either as a sequence of 3‑D points or as a matrix
    node = fs.getNode("grid_points")
    if node.isSeq():
        vals = [float(node.at(i).real()) for i in range(node.size())]
        grid = np.array(vals, dtype=np.float32).reshape(-1, 3)
    else:
        grid = node.mat().astype(np.float32).reshape(-1, 3)
    K = fs.getNode("camera_matrix").mat().astype(np.float64)
    w = int(fs.getNode("image_width").real())
    h = int(fs.getNode("image_height").real())
    bw = int(fs.getNode("board_width").real())
    bh = int(fs.getNode("board_height").real())
    fs.release()
    return errs, extr, grid, K, (w, h), (bw, bh)


def invert_extrinsic(
    rvec: np.ndarray, tvec: np.ndarray
) -> Tuple[np.ndarray, np.ndarray]:
    R, _ = cv2.Rodrigues(rvec)
    R_inv = R.T
    t_inv = -R_inv @ tvec.reshape(3)
    return R_inv, t_inv


def draw_frustum(
    ax,
    K: np.ndarray,
    imsize: Tuple[int, int],
    scale: float,
    pose: Optional[Tuple[np.ndarray, np.ndarray]] = None,
    color: str = "cyan",
    alpha: float = 0.1,
) -> None:
    fx, fy = K[0, 0], K[1, 1]
    cx, cy = K[0, 2], K[1, 2]
    w, h = imsize
    # define a near and far plane for the pyramid
    near = scale
    far = scale * 3.0
    # compute  the 8 vertices of the frustum in camera coordinates
    pts = []
    for d in (near, far):
        for u, v in ((0, 0), (w, 0), (w, h), (0, h)):
            x = (u - cx) / fx * d
            y = (v - cy) / fy * d
            pts.append([x, y, d])
    P = np.array(pts, dtype=np.float64)
    # apply pose transformation if provided
    if pose is not None:
        R_pose, t_pose = pose
        P = (R_pose @ P.T).T + t_pose.reshape(1, 3)
        origin = t_pose.reshape(3)
    else:
        origin = np.zeros(3)
    # define edges of the frustum
    for i in range(4):
        ax.plot(*zip(origin, P[i]), color=color, lw=1)
        ax.plot(*zip(origin, P[i + 4]), color=color, lw=1)
        ax.plot(*zip(P[i], P[(i + 1) % 4]), color=color, lw=1)
        ax.plot(*zip(P[i + 4], P[4 + (i + 1) % 4]), color=color, lw=1)
    for indices in ([0, 1, 2, 3], [4, 5, 6, 7]):
        quad = P[list(indices)]
        ax.add_collection3d(
            Poly3DCollection(
                [quad],
                facecolors=color,
                edgecolors="white",
                linewidths=0.5,
                alpha=alpha,
            )
        )


def build_board_mesh(grid: np.ndarray, board_width: int, board_height: int) -> List[List[np.ndarray]]:
    pts = grid.reshape((board_width, board_height, 3), order="F")
    faces: List[List[np.ndarray]] = []
    for ix in range(board_width - 1):
        for iy in range(board_height - 1):
            p00 = pts[ix, iy]
            p10 = pts[ix + 1, iy]
            p11 = pts[ix + 1, iy + 1]
            p01 = pts[ix, iy + 1]
            faces.append([p00, p10, p11, p01])
    return faces


def plot_scene_camera_view(
    ax,
    extr: np.ndarray,
    grid: np.ndarray,
    K: np.ndarray,
    imsize: Tuple[int, int],
    board_size: Tuple[int, int],
    errors: Optional[List[float]] = None,
    colormap: Optional[callable] = None,
) -> List[Poly3DCollection]:
    Rts: List[Tuple[np.ndarray, np.ndarray]] = []
    for row in extr:
        rvec = row[:3].reshape(3, 1)
        tvec = row[3:].reshape(3, 1)
        R, _ = cv2.Rodrigues(rvec)
        Rts.append((R, tvec.reshape(3)))
    # get the plot limits based on transformed board points
    all_points = []
    for R, t in Rts:
        pts_h = (R @ grid.T).T + t.reshape(1, 3)
        all_points.append(pts_h)
    all_points_arr = np.concatenate(all_points, axis=0)
    mins = all_points_arr.min(axis=0)
    maxs = all_points_arr.max(axis=0)
    spans = maxs - mins
    margin = 0.1
    mins -= spans * margin
    maxs += spans * margin
    ax.set_xlim(mins[0], maxs[0])
    # invert the y axis for camera view so that boards appear upright
    ax.set_ylim(maxs[1], mins[1])
    ax.set_zlim(mins[2], maxs[2])
    ax.set_box_aspect((maxs[0] - mins[0], maxs[1] - mins[1], maxs[2] - mins[2]))
    cmap = colormap or plt.get_cmap("tab20")
    faces_template = build_board_mesh(grid, board_size[0], board_size[1])
    meshes: List[Poly3DCollection] = []
    for idx, (R, t) in enumerate(Rts):
        faces_transformed: List[List[np.ndarray]] = []
        for face in faces_template:
            transformed = [(R @ v.reshape(3, 1)).flatten() + t for v in face]
            faces_transformed.append(transformed)
        mesh = Poly3DCollection(
            faces_transformed, alpha=0.3, linewidths=0.2, picker=True
        )
        mesh.set_facecolor(cmap(idx % 20))
        mesh.set_edgecolor("white")
        ax.add_collection3d(mesh)
        meshes.append(mesh)
        board_centroid = np.mean(
            [(R @ v.reshape(3, 1)).flatten() + t for v in grid], axis=0
        )
        ax.text(*board_centroid, str(idx), color="white", fontsize=8)
    scale = spans.max() * 0.05
    draw_frustum(ax, K, imsize, scale=scale)
    L = scale
    ax.quiver(0, 0, 0, L, 0, 0, color="r", length=L)
    ax.quiver(0, 0, 0, 0, -L, 0, color="g", length=L)
    ax.quiver(0, 0, 0, 0, 0, L, color="b", length=L)
    ax.text(L, 0, 0, "Xc", color="r", fontsize=10)
    ax.text(0, -L, 0, "Yc", color="g", fontsize=10)
    ax.text(0, 0, L, "Zc", color="b", fontsize=10)
    ax.grid(False)
    for axis in (ax.xaxis, ax.yaxis, ax.zaxis):
        axis.pane.fill = False
    ax.set_xlabel("X (mm)", color="white")
    ax.set_ylabel("Y (mm)", color="white")
    ax.set_zlabel("Z (mm)", color="white")
    ax.tick_params(colors="white")
    # set a pleasing viewing angle
    ax.view_init(elev=20, azim=45)
    return meshes


def plot_scene_board_view(
    ax,
    calibs: List[Dict[str, object]],
    board_size: Tuple[int, int],
    colormap: Optional[callable] = None,
    max_frustums_per_cam: Optional[int] = None,
) -> None:
    # determine bounding box across all camera poses
    camera_positions = []
    board_points = calibs[0][
        "grid"
    ]  # assumes that all cameras use the same board pattern
    for calib in calibs:
        extr = calib["extr"]
        for i, row in enumerate(extr):
            # optional subsample
            if max_frustums_per_cam is not None and i >= max_frustums_per_cam:
                break
            rvec = row[:3]
            tvec = row[3:]
            R_inv, t_inv = invert_extrinsic(rvec, tvec)
            camera_positions.append(t_inv)
    camera_positions_arr = np.array(camera_positions)
    mins = np.min(np.vstack((camera_positions_arr, board_points)), axis=0)
    maxs = np.max(np.vstack((camera_positions_arr, board_points)), axis=0)
    spans = maxs - mins
    margin = 0.1
    mins -= spans * margin
    maxs += spans * margin
    ax.set_xlim(mins[0], maxs[0])
    ax.set_ylim(maxs[1], mins[1])
    ax.set_zlim(mins[2], maxs[2])
    ax.set_box_aspect((maxs[0] - mins[0], maxs[1] - mins[1], maxs[2] - mins[2]))
    cmap = colormap or plt.get_cmap("tab10")
    faces = build_board_mesh(board_points, board_size[0], board_size[1])
    board_mesh = Poly3DCollection(faces, alpha=0.3, linewidths=0.2)
    board_mesh.set_facecolor((0.2, 0.8, 1.0, 0.3))
    board_mesh.set_edgecolor("white")
    ax.add_collection3d(board_mesh)
    L = spans.max() * 0.05
    ax.quiver(0, 0, 0, L, 0, 0, color="r", length=L)
    ax.quiver(0, 0, 0, 0, L, 0, color="g", length=L)
    ax.quiver(0, 0, 0, 0, 0, L, color="b", length=L)
    ax.text(L, 0, 0, "Xb", color="r", fontsize=10)
    ax.text(0, L, 0, "Yb", color="g", fontsize=10)
    ax.text(0, 0, L, "Zb", color="b", fontsize=10)
    for cam_idx, calib in enumerate(calibs):
        extr = calib["extr"]
        K = calib["K"]
        imsize = calib["imsize"]
        label = calib.get("label", f"Cam{cam_idx}")
        colour = cmap(cam_idx % 10)
        scale = spans.max() * 0.05
        for i, row in enumerate(extr):
            if max_frustums_per_cam is not None and i >= max_frustums_per_cam:
                break
            rvec = row[:3]
            tvec = row[3:]
            R_inv, t_inv = invert_extrinsic(rvec, tvec)
            draw_frustum(
                ax, K, imsize, scale=scale, pose=(R_inv, t_inv), color=colour, alpha=0.1
            )
            # draw label at camera position for first frustum only
            if i == 0:
                ax.text(*t_inv, label, color=colour, fontsize=9)
    ax.grid(False)
    for axis in (ax.xaxis, ax.yaxis, ax.zaxis):
        axis.pane.fill = False
    ax.set_xlabel("X (board mm)", color="white")
    ax.set_ylabel("Y (board mm)", color="white")
    ax.set_zlabel("Z (board mm)", color="white")
    ax.tick_params(colors="white")
    ax.view_init(elev=20, azim=45)


def export_scene_to_obj_multi(
    base_name: str,
    calibs: List[Dict[str, object]],
    board_size: Tuple[int, int],
    view: str = "camera",
    max_frustums_per_cam: Optional[int] = None,
) -> None:
    # prepare the data structures for obj/mrl
    vertices: List[Tuple[float, float, float]] = []
    faces: List[List[int]] = []
    materials: List[Tuple[str, Tuple[float, float, float], float]] = []
    group_info: List[Tuple[str, int, int, str]] = []
    if view == "camera":
        # first calibration is used in camera view
        calib = calibs[0]
        grid = calib["grid"]
        extr = calib["extr"]
        board_mat = "board_mat"
        materials.append((board_mat, (0.2, 0.8, 1.0), 0.3))
        start_faces = len(faces)
        for row in extr:
            rvec = row[:3]
            tvec = row[3:]
            R, _ = cv2.Rodrigues(rvec)
            pts_h = (R @ grid.T).T + tvec.reshape(1, 3)
            world = pts_h.reshape((board_size[0], board_size[1], 3), order="F")
            corners = [
                world[0, 0],
                world[board_size[0] - 1, 0],
                world[board_size[0] - 1, board_size[1] - 1],
                world[0, board_size[1] - 1],
            ]
            v0 = len(vertices)
            vertices.extend([tuple(v.tolist()) for v in corners])
            faces.append([v0 + 1, v0 + 2, v0 + 3, v0 + 4])
        count = len(faces) - start_faces
        group_info.append(("all_boards", start_faces, count, board_mat))
        frustum_mat = "frustum_mat"
        materials.append((frustum_mat, (1.0, 0.4, 0.2), 1.0))
        start_faces = len(faces)
        K = calib["K"]
        imsize = calib["imsize"]
        board_extent = np.max(np.ptp(grid, axis=0))
        near = 0.05 * board_extent
        far = near * 3
        fx, fy = K[0, 0], K[1, 1]
        cx, cy = K[0, 2], K[1, 2]
        w, h = imsize
        pts_cam: List[List[float]] = []
        for d in (near, far):
            for u, v in ((0, 0), (w, 0), (w, h), (0, h)):
                x = (u - cx) / fx * d
                y = (v - cy) / fy * d
                pts_cam.append([x, y, d])
        pts_cam_arr = np.array(pts_cam)
        apex_idx = len(vertices)
        vertices.append((0.0, 0.0, 0.0))
        start_verts = len(vertices)
        vertices.extend([tuple(p.tolist()) for p in pts_cam_arr])
        for i in range(4):
            faces.append(
                [apex_idx + 1, start_verts + i + 1, start_verts + ((i + 1) % 4) + 1]
            )
        quad = [start_verts + i + 1 for i in range(4, 8)]
        faces.append(quad)
        count = len(faces) - start_faces
        group_info.append(("camera_frustum", start_faces, count, frustum_mat))
    else:
        # single mesh
        board_points = calibs[0]["grid"]
        board_mat = "board_mat"
        materials.append((board_mat, (0.2, 0.8, 1.0), 0.3))
        start_faces = len(faces)
        # use only the outer quad of the board
        ux = np.unique(board_points[:, 0])
        uy = np.unique(board_points[:, 1])
        nx, ny = ux.size, uy.size
        world = board_points.reshape((nx, ny, 3), order="F")
        corners = [
            world[0, 0],
            world[nx - 1, 0],
            world[nx - 1, ny - 1],
            world[0, ny - 1],
        ]
        v0 = len(vertices)
        vertices.extend([tuple(v.tolist()) for v in corners])
        faces.append([v0 + 1, v0 + 2, v0 + 3, v0 + 4])
        count = len(faces) - start_faces
        group_info.append(("board", start_faces, count, board_mat))
        frustum_mat = "frustum_mat"
        materials.append((frustum_mat, (1.0, 0.4, 0.2), 1.0))
        start_faces = len(faces)
        for calib in calibs:
            extr = calib["extr"]
            K = calib["K"]
            imsize = calib["imsize"]
            board_extent = np.max(np.ptp(board_points, axis=0))
            near = 0.05 * board_extent
            far = near * 3
            fx, fy = K[0, 0], K[1, 1]
            cx, cy = K[0, 2], K[1, 2]
            w, h = imsize
            # compute camera pose only from the first extrinsic entry per camera
            if extr.shape[0] == 0:
                continue
            rvec = extr[0, :3]
            tvec = extr[0, 3:]
            R_inv, t_inv = invert_extrinsic(rvec, tvec)
            pts_cam: List[List[float]] = []
            for d in (near, far):
                for u, v in ((0, 0), (w, 0), (w, h), (0, h)):
                    x = (u - cx) / fx * d
                    y = (v - cy) / fy * d
                    pts_cam.append([x, y, d])
            pts_cam_arr = np.array(pts_cam)
            world_pts = (R_inv @ pts_cam_arr.T).T + t_inv.reshape(1, 3)
            apex_idx = len(vertices)
            vertices.append(tuple(t_inv.tolist()))
            start_verts = len(vertices)
            vertices.extend([tuple(p.tolist()) for p in world_pts])
            # pyramid sides
            for i in range(4):
                faces.append(
                    [
                        apex_idx + 1,
                        start_verts + i + 1,
                        start_verts + ((i + 1) % 4) + 1,
                    ]
                )
            # far quad
            quad = [start_verts + i + 1 for i in range(4, 8)]
            faces.append(quad)
        count = len(faces) - start_faces
        group_info.append(("camera_frustums", start_faces, count, frustum_mat))
    # write the mtl file
    mtl_path = base_name + ".mtl"
    with open(mtl_path, "w") as f_mtl:
        for name, colour, alpha in materials:
            f_mtl.write(f"newmtl {name}\n")
            f_mtl.write(f"Kd {colour[0]:.3f} {colour[1]:.3f} {colour[2]:.3f}\n")
            f_mtl.write(f"d {alpha:.2f}\nKa 0 0 0\nKs 0 0 0\n\n")
    # write obj file
    obj_path = base_name + ".obj"
    with open(obj_path, "w") as f_obj:
        f_obj.write(f"mtllib {base_name}.mtl\n")
        # vertices
        for v in vertices:
            f_obj.write(f"v {v[0]:.6f} {v[1]:.6f} {v[2]:.6f}\n")
        # groups
        for group, start, cnt, mat in group_info:
            f_obj.write(f"g {group}\nusemtl {mat}\n")
            for fi in range(start, start + cnt):
                face = faces[fi]
                # obj indices are 1‑based
                f_obj.write("f " + " ".join(str(idx) for idx in face) + "\n")
    print(
        f"Exported scene to {obj_path} (vertices={len(vertices)}, faces={len(faces)})"
    )


def main(argv: Optional[Iterable[str]] = None) -> int:
    # parse command line options
    parser = argparse.ArgumentParser(
        description="Visualise OpenCV calibration in 3‑D and export to OBJ"
    )
    parser.add_argument("calib_files", nargs="+", help="Calibration YAML files")
    parser.add_argument(
        "--view",
        choices=["camera", "board"],
        default=None,
        help="Reference frame: 'camera' shows board relative to a single camera, 'board' shows cameras around the board."
        "The default is 'camera' for a single file and 'board' for multiple files.",
    )
    parser.add_argument(
        "--export", metavar="BASE", help="Export scene as BASE.obj/BASE.mtl for MeshLab"
    )
    parser.add_argument(
        "--no-gui", action="store_true", help="Do not open the interactive viewer"
    )
    parser.add_argument(
        "--max-frustums",
        type=int,
        default=None,
        help="Limit the number of frustums per camera when exporting or drawing (board view only)",
    )
    args = parser.parse_args(argv)
    view = args.view
    if view is None:
        view = "camera" if len(args.calib_files) == 1 else "board"
    calibs: List[Dict[str, object]] = []
    for idx, fname in enumerate(args.calib_files):
        errs, extr, grid, K, imsize, board_size = load_calib(fname)
        calib_data: Dict[str, object] = {
            "errors": errs,
            "extr": extr,
            "grid": grid,
            "K": K,
            "imsize": imsize,
            "board_size": board_size,
            "label": f"Cam{idx}",
        }
        calibs.append(calib_data)
    # export if requested
    if args.export:
        export_scene_to_obj_multi(
            args.export,
            calibs,
            board_size,
            view=view,
            max_frustums_per_cam=args.max_frustums,
        )
    # interactive plot can be displayed optionally
    if not args.no_gui:
        plt.style.use("dark_background")
        if len(calibs) == 1 and view == "camera":
            fig = plt.figure(figsize=(14, 6))
            fig.patch.set_facecolor("black")
            ax_err = fig.add_subplot(1, 2, 1)
            ax_3d = fig.add_subplot(1, 2, 2, projection="3d")
            ax_3d.set_facecolor("black")
            # Plot error bars
            errors = calibs[0]["errors"]
            idxs = np.arange(1, len(errors) + 1)
            bars = ax_err.bar(
                idxs, errors, picker=5, color="#00BCD4", edgecolor="white"
            )
            m = np.mean(errors)
            ax_err.axhline(
                m, color="#FF5722", linestyle="--", label=f"Mean = {m:.3f}px"
            )
            ax_err.set_xlabel("Image index", color="white")
            ax_err.set_ylabel("Error (px)", color="white")
            ax_err.set_title("Per-view RMS Reprojection Error", color="white")
            ax_err.tick_params(colors="white")
            ax_err.legend(loc="upper right", facecolor="black", edgecolor="white")
            meshes = plot_scene_camera_view(
                ax_3d,
                calibs[0]["extr"],
                calibs[0]["grid"],
                calibs[0]["K"],
                calibs[0]["imsize"],
                calibs[0]["board_size"],
                errors=calibs[0]["errors"],
            )

            def onselect_y(vmin, vmax):
                threshold = min(vmin, vmax)
                for i, bar in enumerate(bars):
                    if bar.get_height() >= threshold:
                        bar.set_color("#FFEB3B")
                        bar.set_edgecolor("yellow")
                        meshes[i].set_alpha(0.8)
                        meshes[i].set_edgecolor("yellow")
                    else:
                        bar.set_color("#00BCD4")
                        bar.set_edgecolor("white")
                        meshes[i].set_alpha(0.1)
                        meshes[i].set_edgecolor("white")
                fig.canvas.draw_idle()

            SpanSelector(
                ax_err,
                onselect_y,
                "vertical",
                useblit=True,
                props=dict(alpha=0.3, facecolor="yellow"),
                minspan=0,
            )

            def on_pick(event):
                artist = event.artist
                if artist in bars:
                    idx = list(bars).index(artist)
                elif artist in meshes:
                    idx = meshes.index(artist)
                else:
                    return
                for j, bar in enumerate(bars):
                    if j == idx:
                        bar.set_color("#FFEB3B")
                        bar.set_edgecolor("yellow")
                        meshes[j].set_alpha(0.8)
                        meshes[j].set_edgecolor("yellow")
                    else:
                        bar.set_color("#00BCD4")
                        bar.set_edgecolor("white")
                        meshes[j].set_alpha(0.1)
                        meshes[j].set_edgecolor("white")
                fig.canvas.draw_idle()

            fig.canvas.mpl_connect("pick_event", on_pick)
            plt.tight_layout()
            plt.show()
        else:
            fig = plt.figure(figsize=(8, 6))
            fig.patch.set_facecolor("black")
            ax_3d = fig.add_subplot(1, 1, 1, projection="3d")
            ax_3d.set_facecolor("black")
            if view == "camera":
                # single camera without error bar
                plot_scene_camera_view(
                    ax_3d,
                    calibs[0]["extr"],
                    calibs[0]["grid"],
                    calibs[0]["K"],
                    calibs[0]["imsize"],
                    calibs[0]["board_size"]
                )
            else:
                # board view for multiple cameras
                plot_scene_board_view(
                    ax_3d,
                    calibs,
                    calibs[0]["board_size"],
                    max_frustums_per_cam=args.max_frustums,
                )
            plt.tight_layout()
            plt.show()
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
import sys
import cv2 as cv
import argparse
from common import *

def help():
    print(
        '''
        Use this script for testing Object Tracking using OpenCV.
        Firstly, download required models using the download_models.py.
        To run:
            nanotrack:
                Download Model: python download_models.py nanotrack
                Example: python object_tracker.py nanotrack
            vit:
                Download Model: python download_models.py vit
                Example: python object_tracker.py vit
                                or
                        python object_tracker.py
            dasiamrpn:
                Download Model: python download_models.py dasiamrpn
                Example: python object_tracker.py dasiamrpn
        To switch between models in runtime, make sure all the models are downloaded using download_models.py'''
    )

def load_parser(model_name):
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--zoo', default=os.path.join(os.path.dirname(os.path.abspath(__file__)), 'models.yml'),
                        help='An optional path to file with preprocessing parameters.')
    parser.add_argument("--input", type=str, help="Path to video source")
    args, _ = parser.parse_known_args()

    add_preproc_args(args.zoo, parser, 'object_tracker', alias=model_name)
    if model_name == "dasiamrpn":
        add_preproc_args(args.zoo, parser, 'object_tracker', prefix="dasiamrpn_", alias="dasiamrpn")
        add_preproc_args(args.zoo, parser, 'object_tracker', prefix="dasiamrpn_kernel_r1_", alias="dasiamrpn")
        add_preproc_args(args.zoo, parser, 'object_tracker', prefix="dasiamrpn_kernel_cls_", alias="dasiamrpn")
    elif model_name == "nanotrack":
        add_preproc_args(args.zoo, parser, 'object_tracker', prefix="nanotrack_back_", alias="nanotrack")
        add_preproc_args(args.zoo, parser, 'object_tracker', prefix="nanotrack_head_", alias="nanotrack")
    elif model_name != "vit":
        print("Pass the valid alias. Choices are { nanotrack, vit, dasiamrpn }")
        exit(0)
    parser = argparse.ArgumentParser(parents=[parser],
                                    description='''
    Firstly, download required models using `python download_models.py {modelName}`
    Run using python object_tracker.py {modelName}.
    ''',
                                    formatter_class=argparse.RawTextHelpFormatter)
    return parser.parse_args()

def createTracker(model_name, args):
    if model_name == 'dasiamrpn':
        print("Using Dasiamrpn Tracker.")
        params = cv.TrackerDaSiamRPN_Params()
        params.model = findModel(args.dasiamrpn_model, args.dasiamrpn_sha1)
        params.kernel_cls1 = findModel(args.dasiamrpn_kernel_cls_model, args.dasiamrpn_kernel_cls_sha1)
        params.kernel_r1 = findModel(args.dasiamrpn_kernel_r1_model, args.dasiamrpn_kernel_r1_sha1)
        tracker = cv.TrackerDaSiamRPN_create(params)
    elif model_name == 'nanotrack':
        print("Using Nano Tracker.")
        params = cv.TrackerNano_Params()
        params.backbone = findModel(args.nanotrack_back_model, args.nanotrack_back_sha1)
        params.neckhead = findModel(args.nanotrack_head_model, args.nanotrack_head_sha1)
        tracker = cv.TrackerNano_create(params)
    elif model_name == 'vit':
        print("Using Vit Tracker.")
        params = cv.TrackerVit_Params()
        params.net = findModel(args.model, args.sha1)
        tracker = cv.TrackerVit_create(params)
    else:
        help()
        exit(-1)
    return tracker

def main(model_name, args):
    tracker = createTracker(model_name, args)
    videoPath = args.input
    print('Using video: {}'.format(videoPath))
    cap = cv.VideoCapture(cv.samples.findFile(args.input) if args.input else 0)
    if not cap.isOpened():
        print("Can't open video stream: {}".format(videoPath))
        exit(-1)

    stdSize = 0.6
    stdWeight = 2
    stdImgSize = 512
    imgWidth = -1 # Initialization
    fontSize = 1.5
    fontThickness = 1
    alpha = 0.5
    windowName = "TRACKING"
    cv.namedWindow(windowName, cv.WINDOW_NORMAL)

    while True:
        ret, image = cap.read()
        if not ret:
            print("Video completed!!")
            return -1
        if imgWidth == -1:
            imgWidth = min(image.shape[:2])
            fontSize = min(fontSize, (stdSize*imgWidth)/stdImgSize)
            fontThickness = max(fontThickness,(stdWeight*imgWidth)//stdImgSize)
            label = "Press space bar to pause video to draw bounding box."
            labelSize, _ = cv.getTextSize(label, cv.FONT_HERSHEY_SIMPLEX, fontSize, fontThickness)
        org_img = image.copy()
        cv.rectangle(image, (0, 0), (labelSize[0]+10, labelSize[1]+int(40*fontSize)), (255,255,255), cv.FILLED)
        cv.addWeighted(image, alpha, org_img, 1 - alpha, 0, image)
        cv.putText(image, label, (10, int(25*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
        cv.putText(image, "Press space bar after selecting.", (10, int(55*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
        cv.imshow(windowName, image)

        key = cv.waitKey(30) & 0xFF
        if key == ord(' '):
            bbox = cv.selectROI(windowName, image)
            print('ROI: {}'.format(bbox))
            if bbox != (0, 0, 0, 0):
                break

        if key == ord('q') or key == 27:
            return
    try:
        tracker.init(image, bbox)
    except Exception as e:
        print('Unable to initialize tracker with requested bounding box. Is there any object?')
        print(e)

    tick_meter = cv.TickMeter()
    while cap.isOpened():
        ret, frame = cap.read()
        if not ret:
            break
        if imgWidth == -1:
            imgWidth = min(frame.shape[:2])
            fontSize = min(fontSize, (stdSize*imgWidth)/stdImgSize)
            fontThickness = max(fontThickness,(stdWeight*imgWidth)//stdImgSize)
            label="Press space bar to select new target"
            labelSize, _ = cv.getTextSize(label, cv.FONT_HERSHEY_SIMPLEX, fontSize, fontThickness)
        tick_meter.reset()
        tick_meter.start()
        ok, newbox = tracker.update(frame)
        tick_meter.stop()
        score = tracker.getTrackingScore()
        render_image = frame.copy()
        key = cv.waitKey(30) & 0xFF
        h, w = frame.shape[:2]
        cv.rectangle(render_image, (0, 0), (labelSize[0]+10, labelSize[1]+int(100*fontSize)), (255,255,255), cv.FILLED)
        cv.rectangle(render_image, (0, int(h-45*fontSize)), (w, h), (255,255,255), cv.FILLED)
        cv.addWeighted(render_image, alpha, frame, 1 - alpha, 0, render_image)
        cv.putText(render_image, label, (10, int(25*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
        cv.putText(render_image, "For switching between trackers: press 'v' for ViT, 'n' for Nanotrack, and 'd' for DaSiamRPN.", (10, h-10), cv.FONT_HERSHEY_SIMPLEX, 0.8*fontSize, (0, 0, 0), fontThickness)

        if ok:
            if key == ord(' '):
                cv.putText(render_image, "Select the new target", (10, int(55*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
                bbox = cv.selectROI(windowName, render_image)
                print('ROI:', bbox)
                if bbox != (0, 0, 0, 0):
                    tracker.init(frame, bbox)
            elif key == ord('v'):
                model_name = "vit"
                args = load_parser(model_name)
                tracker = createTracker(model_name, args)
                tracker.init(frame, newbox)
            elif key == ord('n'):
                model_name = "nanotrack"
                args = load_parser(model_name)
                tracker = createTracker(model_name, args)
                tracker.init(frame, newbox)
            elif key == ord('d'):
                model_name = "dasiamrpn"
                args = load_parser(model_name)
                tracker = createTracker(model_name, args)
                tracker.init(frame, newbox)
            elif key == ord('q') or key == 27:
                return

            cv.rectangle(render_image, newbox, (200, 0, 0), thickness=2)
        time_label = f"FPS: {tick_meter.getFPS():.2f}"
        score_label = f"Tracking score: {score:.2f}"
        algo_label = f"Algorithm: {model_name}"
        cv.putText(render_image, time_label, (10, int(55*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
        cv.putText(render_image, score_label, (10, int(85*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
        cv.putText(render_image, algo_label, (10, int(115*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)

        cv.imshow(windowName, render_image)
        if key in [ord('q'), 27]:
            break

if __name__ == '__main__':
    help()
    if len(sys.argv) < 2 or sys.argv[1].startswith("--"):
        model_name = "vit"
    else:
        model_name = sys.argv[1]
    args = load_parser(model_name)

    main(model_name, args)
    cv.destroyAllWindows()



#!/usr/bin/env python3
# This file is part of OpenCV project.
# It is subject to the license terms in the LICENSE file found in the top-level directory
# of this distribution and at http://opencv.org/license.html

import argparse
import sys
import cv2 as cv

USAGE = """\
Chromatic Aberration Correction Sample
Usage:
  chromatic_aberration_correction.py <input_image> <calibration_file> [--bayer <code>] [--output <path>]

Arguments:
  input_image       Path to the input image. Can be:
                      • a 3-channel BGR image, or
                      • a 1-channel raw Bayer image (see bayer_pattern)
  calibration_file  OpenCV YAML/XML file with chromatic aberration calibration:
                      image_width, image_height, red_channel/coeffs_x, coeffs_y,
                      blue_channel/coeffs_x, coeffs_y.
  output            (optional) Path to save the corrected image. Default: corrected.png
  bayer             (optional) integer code for demosaicing a 1-channel raw image
                    If omitted or <0, input is assumed 3-channel BGR.

Example:
  python chromatic_aberration_correction.py input.png calib.yaml --bayer 46 --output corrected.png
"""

def main(argv=None):
    parser = argparse.ArgumentParser(
        description="Chromatic Aberration Correction Sample",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=USAGE
    )
    parser.add_argument("input", help="Input image (BGR or Bayer)")
    parser.add_argument("calibration", help="Calibration file (YAML/XML)")
    parser.add_argument("--output", default="corrected.png", help="Output image file")
    parser.add_argument("--bayer", type=int, default=-1, help="Bayer pattern code for demosaic")
    parser.add_argument("--no-gui", action="store_true", help="Do not open image windows")

    args = parser.parse_args(argv)

    img = cv.imread(args.input, cv.IMREAD_UNCHANGED)
    if img is None:
        print(f"ERROR: Could not load input image: {args.input}", file=sys.stderr)
        return 1

    fs = cv.FileStorage(args.calibration, cv.FileStorage_READ)
    if not fs.isOpened():
        print(f"Could not calibration coefficients from {args.calibration}")
        return 1

    try:
        coeffMat, size, degree = cv.loadChromaticAberrationParams(fs.root())
        corrected = cv.correctChromaticAberration(img, coeffMat, size, degree, args.bayer)

        if corrected is None:
            print("ERROR: cv.correctChromaticAberration returned None", file=sys.stderr)
            return 1

        if not args.no_gui:
            cv.namedWindow("Original", cv.WINDOW_AUTOSIZE)
            cv.namedWindow("Corrected", cv.WINDOW_AUTOSIZE)
            cv.imshow("Original", img)
            cv.imshow("Corrected", corrected)
            print("Press any key to continue...")
            cv.waitKey(0)
            cv.destroyAllWindows()

        if not cv.imwrite(args.output, corrected):
            print(f"WARNING: Could not write output image: {args.output}", file=sys.stderr)
        else:
            print(f"Saved corrected image to: {args.output}")

    except cv.error as e:
        print(f"OpenCV error: {e}", file=sys.stderr)
        return 1

    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
'''
This file is part of OpenCV project.
It is subject to the license terms in the LICENSE file found in the top-level directory
of this distribution and at http://opencv.org/license.html.

This sample inpaints the masked area in the given image.

Copyright (C) 2025, Bigvision LLC.

How to use:
    Sample command to run:
        `python inpainting.py`
    The system will ask you to draw the mask to be inpainted

    You can download lama inpainting model using
        `python download_models.py lama`

    References:
      Github: https://github.com/advimman/lama
      ONNX model: https://huggingface.co/Carve/LaMa-ONNX/blob/main/lama_fp32.onnx

      ONNX model was further quantized using block quantization from [opencv_zoo](https://github.com/opencv/opencv_zoo)

    Set environment variable OPENCV_DOWNLOAD_CACHE_DIR to point to the directory where models are downloaded. Also, point OPENCV_SAMPLES_DATA_PATH to opencv/samples/data.
'''
import argparse
import os.path
import numpy as np
import cv2 as cv
from common import *

def help():
    print(
        '''
        Use this script for image inpainting using OpenCV.

        Firstly, download required models i.e. lama using `download_models.py` (if not already done). Set environment variable OPENCV_DOWNLOAD_CACHE_DIR to specify where models should be downloaded. Also, point OPENCV_SAMPLES_DATA_PATH to opencv/samples/data.

        To run:
        Example: python inpainting.py [--input=<image_name>]

        Inpainting model path can also be specified using --model argument.
        '''
    )

def keyboard_shorcuts():
    print('''
    Keyboard Shorcuts:
        Press 'i' to increase brush size.
        Press 'd' to decrease brush size.
        Press 'r' to reset mask.
        Press ' ' (space bar) after selecting area to be inpainted.
        Press ESC to terminate the program.
    '''
    )

def get_args_parser():
    backends = ("default", "openvino", "opencv", "vkcom", "cuda")
    targets = ("cpu", "opencl", "opencl_fp16", "ncs2_vpu", "hddl_vpu", "vulkan", "cuda", "cuda_fp16")

    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--zoo', default=os.path.join(os.path.dirname(os.path.abspath(__file__)), 'models.yml'),
                        help='An optional path to file with preprocessing parameters.')
    parser.add_argument('--input', '-i', default="rubberwhale1.png", help='Path to image file.', required=False)
    parser.add_argument('--backend', default="default", type=str, choices=backends,
            help="Choose one of computation backends: "
            "default: automatically (by default), "
            "openvino: Intel's Deep Learning Inference Engine (https://software.intel.com/openvino-toolkit), "
            "opencv: OpenCV implementation, "
            "vkcom: VKCOM, "
            "cuda: CUDA, "
            "webnn: WebNN")
    parser.add_argument('--target', default="cpu", type=str, choices=targets,
            help="Choose one of target computation devices: "
            "cpu: CPU target (by default), "
            "opencl: OpenCL, "
            "opencl_fp16: OpenCL fp16 (half-float precision), "
            "ncs2_vpu: NCS2 VPU, "
            "hddl_vpu: HDDL VPU, "
            "vulkan: Vulkan, "
            "cuda: CUDA, "
            "cuda_fp16: CUDA fp16 (half-float preprocess)")
    args, _ = parser.parse_known_args()
    add_preproc_args(args.zoo, parser, 'inpainting', prefix="", alias="lama")
    parser = argparse.ArgumentParser(parents=[parser],
                                        description='Image inpainting using OpenCV.',
                                        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    return parser.parse_args()


drawing = False
mask_gray = None
brush_size = 15

def draw_mask(event, x, y, flags, param):
    global drawing, mask_gray, brush_size
    if event == cv.EVENT_LBUTTONDOWN:
        drawing = True
    elif event == cv.EVENT_MOUSEMOVE:
        if drawing:
            cv.circle(mask_gray, (x, y), brush_size, (255), thickness=-1)
    elif event == cv.EVENT_LBUTTONUP:
        drawing = False

def main():
    global mask_gray, brush_size

    print("Model loading...")

    if hasattr(args, 'help'):
        help()
        exit(1)

    args.model = findModel(args.model, args.sha1)

    engine = cv.dnn.ENGINE_AUTO

    if args.backend != "default" or args.target != "cpu":
        engine = cv.dnn.ENGINE_CLASSIC

    net = cv.dnn.readNetFromONNX(args.model, engine)
    net.setPreferableBackend(get_backend_id(args.backend))
    net.setPreferableTarget(get_target_id(args.target))

    input_image = cv.imread(findFile(args.input))
    aspect_ratio = input_image.shape[0]/input_image.shape[1]
    height = int(args.width*aspect_ratio)

    input_image = cv.resize(input_image, (args.width, height))
    image = input_image.copy()
    keyboard_shorcuts()

    stdSize = 0.7
    stdWeight = 2
    stdImgSize = 512
    imgWidth = min(input_image.shape[:2])
    fontSize = min(1.5, (stdSize*imgWidth)/stdImgSize)
    fontThickness = max(1,(stdWeight*imgWidth)//stdImgSize)

    label = "Press 'i' to increase, 'd' to decrease brush size. And 'r' to reset mask. "
    labelSize, _ = cv.getTextSize(label, cv.FONT_HERSHEY_SIMPLEX, fontSize, fontThickness)
    alpha = 0.5
    # Setting up the window
    cv.namedWindow("Draw Mask")
    cv.setMouseCallback("Draw Mask", draw_mask)
    temp_image = input_image.copy()
    overlay = input_image.copy()
    cv.rectangle(overlay, (0, 0), (labelSize[0]+10, labelSize[1]+int(30*fontSize)), (255, 255, 255), cv.FILLED)
    cv.addWeighted(overlay, alpha, temp_image, 1 - alpha, 0, temp_image)
    cv.putText(temp_image, "Draw the mask on the image. Press space bar when done.", (10, int(25*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
    cv.putText(temp_image, label, (10, int(50*fontSize)), cv.FONT_HERSHEY_SIMPLEX, fontSize, (0, 0, 0), fontThickness)
    display_image = temp_image.copy()

    while True:
        mask_gray = np.zeros((input_image.shape[0], input_image.shape[1]), dtype=np.uint8)
        display_image = temp_image.copy()
        while True:
            display_image[mask_gray > 0] = [255, 255, 255]
            cv.imshow("Draw Mask", display_image)
            key = cv.waitKey(30) & 0xFF
            if key == ord('i'):  # Increase brush size
                brush_size += 1
                print(f"Brush size increased to {brush_size}")
            elif key == ord('d'):  # Decrease brush size
                brush_size = max(1, brush_size - 1)
                print(f"Brush size decreased to {brush_size}")
            elif key == ord('r'):  # clear the mask
                mask_gray = np.zeros((input_image.shape[0], input_image.shape[1]), dtype=np.uint8)
                display_image = temp_image.copy()
                print(f"Mask cleared")
            elif key == ord(' '): # Press space bar to finish drawing
                break
            elif key == 27:
                exit()

        print("Processing image...")
        # Inference block
        image_blob = cv.dnn.blobFromImage(image, args.scale, (args.width, args.height), args.mean, args.rgb, False)
        mask_blob = cv.dnn.blobFromImage(mask_gray, scalefactor=1.0, size=(args.width, args.height), mean=(0,), swapRB=False, crop=False)
        mask_blob = (mask_blob > 0).astype(np.float32)

        net.setInput(image_blob, "image")
        net.setInput(mask_blob, "mask")

        output = net.forward()

        # Postprocessing
        output_image = output[0]
        output_image = np.transpose(output_image, (1, 2, 0))
        output_image = (output_image).astype(np.uint8)
        output_image = cv.resize(output_image, (args.width, height))
        image = output_image

        cv.imshow("Inpainted Output", output_image)

if __name__ == '__main__':
    args = get_args_parser()
    main()


#!/usr/bin/env python
import os
import cv2 as cv
import numpy as np

from tests_common import NewOpenCVTests, unittest

def normAssert(test, a, b, msg=None, lInf=1e-5):
    test.assertLess(np.max(np.abs(a - b)), lInf, msg)

def inter_area(box1, box2):
    x_min, x_max = max(box1[0], box2[0]), min(box1[2], box2[2])
    y_min, y_max = max(box1[1], box2[1]), min(box1[3], box2[3])
    return (x_max - x_min) * (y_max - y_min)

def area(box):
    return (box[2] - box[0]) * (box[3] - box[1])

def box2str(box):
    left, top = box[0], box[1]
    width, height = box[2] - left, box[3] - top
    return '[%f x %f from (%f, %f)]' % (width, height, left, top)

def normAssertDetections(test, refClassIds, refScores, refBoxes, testClassIds, testScores, testBoxes,
                 confThreshold=0.0, scores_diff=1e-5, boxes_iou_diff=1e-4):
    matchedRefBoxes = [False] * len(refBoxes)
    errMsg = ''
    for i in range(len(testBoxes)):
        testScore = testScores[i]
        if testScore < confThreshold:
            continue

        testClassId, testBox = testClassIds[i], testBoxes[i]
        matched = False
        for j in range(len(refBoxes)):
            if (not matchedRefBoxes[j]) and testClassId == refClassIds[j] and \
               abs(testScore - refScores[j]) < scores_diff:
                interArea = inter_area(testBox, refBoxes[j])
                iou = interArea / (area(testBox) + area(refBoxes[j]) - interArea)
                if abs(iou - 1.0) < boxes_iou_diff:
                    matched = True
                    matchedRefBoxes[j] = True
        if not matched:
            errMsg += '\nUnmatched prediction: class %d score %f box %s' % (testClassId, testScore, box2str(testBox))

    for i in range(len(refBoxes)):
        if (not matchedRefBoxes[i]) and refScores[i] > confThreshold:
            errMsg += '\nUnmatched reference: class %d score %f box %s' % (refClassIds[i], refScores[i], box2str(refBoxes[i]))
    if errMsg:
        test.fail(errMsg)

def printParams(backend, target):
    backendNames = {
        cv.dnn.DNN_BACKEND_OPENCV: 'OCV',
        cv.dnn.DNN_BACKEND_INFERENCE_ENGINE: 'DLIE'
    }
    targetNames = {
        cv.dnn.DNN_TARGET_CPU: 'CPU',
        cv.dnn.DNN_TARGET_OPENCL: 'OCL',
        cv.dnn.DNN_TARGET_OPENCL_FP16: 'OCL_FP16',
        cv.dnn.DNN_TARGET_MYRIAD: 'MYRIAD'
    }
    print('%s/%s' % (backendNames[backend], targetNames[target]))

def getDefaultThreshold(target):
    if target == cv.dnn.DNN_TARGET_OPENCL_FP16 or target == cv.dnn.DNN_TARGET_MYRIAD:
        return 4e-3
    else:
        return 1e-5

testdata_required = bool(os.environ.get('OPENCV_DNN_TEST_REQUIRE_TESTDATA', False))

g_dnnBackendsAndTargets = None

class dnn_test(NewOpenCVTests):

    def setUp(self):
        super(dnn_test, self).setUp()

        global g_dnnBackendsAndTargets
        if g_dnnBackendsAndTargets is None:
            g_dnnBackendsAndTargets = self.initBackendsAndTargets()
        self.dnnBackendsAndTargets = g_dnnBackendsAndTargets

    def checkIETarget(self, backend, target):
        # OpenVINO is optional; a target is usable only if its backend lists it.
        try:
            return target in cv.dnn.getAvailableTargets(backend)
        except BaseException:
            return False

    def initBackendsAndTargets(self):
        self.dnnBackendsAndTargets = [
            [cv.dnn.DNN_BACKEND_OPENCV, cv.dnn.DNN_TARGET_CPU],
        ]

        if self.checkIETarget(cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_CPU):
            self.dnnBackendsAndTargets.append([cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_CPU])
        if self.checkIETarget(cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_MYRIAD):
            self.dnnBackendsAndTargets.append([cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_MYRIAD])

        if cv.ocl.haveOpenCL() and cv.ocl.useOpenCL():
            self.dnnBackendsAndTargets.append([cv.dnn.DNN_BACKEND_OPENCV, cv.dnn.DNN_TARGET_OPENCL])
            self.dnnBackendsAndTargets.append([cv.dnn.DNN_BACKEND_OPENCV, cv.dnn.DNN_TARGET_OPENCL_FP16])
            if cv.ocl_Device.getDefault().isIntel():
                if self.checkIETarget(cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_OPENCL):
                    self.dnnBackendsAndTargets.append([cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_OPENCL])
                if self.checkIETarget(cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_OPENCL_FP16):
                    self.dnnBackendsAndTargets.append([cv.dnn.DNN_BACKEND_INFERENCE_ENGINE, cv.dnn.DNN_TARGET_OPENCL_FP16])
        return self.dnnBackendsAndTargets

    def find_dnn_file(self, filename, required=True):
        if not required:
            required = testdata_required
        return self.find_file(filename, [os.environ.get('OPENCV_DNN_TEST_DATA_PATH', os.getcwd()),
                                         os.environ['OPENCV_TEST_DATA_PATH']],
                              required=required)

    def test_getAvailableTargets(self):
        targets = cv.dnn.getAvailableTargets(cv.dnn.DNN_BACKEND_OPENCV)
        self.assertTrue(cv.dnn.DNN_TARGET_CPU in targets)

    def test_blobRectsToImageRects(self):
        paramNet = cv.dnn.Image2BlobParams()
        paramNet.size = (226, 226)
        paramNet.ddepth = cv.CV_32F
        paramNet.mean = [0.485, 0.456, 0.406]
        paramNet.scalefactor = [0.229, 0.224, 0.225]
        paramNet.swapRB = False
        paramNet.datalayout = cv.DATA_LAYOUT_NCHW
        paramNet.paddingmode = cv.dnn.DNN_PMODE_LETTERBOX
        rBlob = np.zeros(shape=(20, 4), dtype=np.int32)
        rImg = paramNet.blobRectsToImageRects(rBlob, (356, 356))
        self.assertTrue(type(rImg[0, 0])==np.int32)
        self.assertTrue(rImg.shape==(20, 4))

    def test_blobRectToImageRect(self):
        paramNet = cv.dnn.Image2BlobParams()
        paramNet.size = (226, 226)
        paramNet.ddepth = cv.CV_32F
        paramNet.mean = [0.485, 0.456, 0.406]
        paramNet.scalefactor = [0.229, 0.224, 0.225]
        paramNet.swapRB = False
        paramNet.datalayout = cv.DATA_LAYOUT_NCHW
        paramNet.paddingmode = cv.dnn.DNN_PMODE_LETTERBOX
        rBlob = np.zeros(shape=(20, 4), dtype=np.int32)
        rImg = paramNet.blobRectToImageRect((0, 0, 0, 0), (356, 356))
        self.assertTrue(type(rImg[0])==int)


    def test_blobFromImage(self):
        np.random.seed(324)

        width = 6
        height = 7
        scale = 1.0/127.5
        mean = (10, 20, 30)

        # Test arguments names.
        img = np.random.randint(0, 255, [4, 5, 3]).astype(np.uint8)
        blob = cv.dnn.blobFromImage(img, scale, (width, height), mean, True, False)
        blob_args = cv.dnn.blobFromImage(img, scalefactor=scale, size=(width, height),
                                         mean=mean, swapRB=True, crop=False)
        normAssert(self, blob, blob_args)

        # Test values.
        target = cv.resize(img, (width, height), interpolation=cv.INTER_LINEAR)
        target = target.astype(np.float32)
        target = target[:,:,[2, 1, 0]]  # BGR2RGB
        target[:,:,0] -= mean[0]
        target[:,:,1] -= mean[1]
        target[:,:,2] -= mean[2]
        target *= scale
        target = target.transpose(2, 0, 1).reshape(1, 3, height, width)  # to NCHW
        normAssert(self, blob, target)

    def test_blobFromImageWithParams(self):
        np.random.seed(324)

        width = 6
        height = 7
        stddev = np.array([0.2, 0.3, 0.4])
        scalefactor = 1.0/127.5 * stddev
        mean = (10, 20, 30)

        # Test arguments names.
        img = np.random.randint(0, 255, [4, 5, 3]).astype(np.uint8)

        param = cv.dnn.Image2BlobParams()
        param.scalefactor = scalefactor
        param.size = (6, 7)
        param.mean = mean
        param.swapRB=True
        param.datalayout = cv.DATA_LAYOUT_NHWC

        blob = cv.dnn.blobFromImageWithParams(img, param)
        blob_args = cv.dnn.blobFromImageWithParams(img, cv.dnn.Image2BlobParams(scalefactor=scalefactor, size=(6, 7), mean=mean,
                                                                      swapRB=True, datalayout=cv.DATA_LAYOUT_NHWC))
        normAssert(self, blob, blob_args)

        target2 = cv.resize(img, (width, height), interpolation=cv.INTER_LINEAR).astype(np.float32)
        target2 = target2[:,:,[2, 1, 0]]  # BGR2RGB
        target2[:,:,0] -= mean[0]
        target2[:,:,1] -= mean[1]
        target2[:,:,2] -= mean[2]

        target2[:,:,0] *= scalefactor[0]
        target2[:,:,1] *= scalefactor[1]
        target2[:,:,2] *= scalefactor[2]
        target2 = target2.reshape(1, height, width, 3)  # to NHWC
        normAssert(self, blob, target2)

    def test_model(self):
        img_path = self.find_dnn_file("dnn/street.png")
        weights = self.find_dnn_file("dnn/onnx/models/ssd_vgg16.onnx", required=False)
        if weights is None:
            raise unittest.SkipTest("Missing DNN test files (dnn/onnx/models/ssd_vgg16.onnx). Verify OPENCV_DNN_TEST_DATA_PATH configuration parameter.")

        frame = cv.imread(img_path)
        model = cv.dnn_DetectionModel(weights)
        model.setInputParams(size=(300, 300), mean=(0, 0, 0), scale=1.0, swapRB=False)

        iouDiff = 0.05
        confThreshold = 0.3
        nmsThreshold = 0
        scoreDiff = 5e-3

        classIds, confidences, boxes = model.detect(frame, confThreshold, nmsThreshold)

        refClassIds = (37,)
        refConfidences = (0.8196,)
        refBoxes = ((331, 233, 85, 107),)

        normAssertDetections(self, refClassIds, refConfidences, refBoxes,
                             classIds, confidences, boxes,confThreshold, scoreDiff, iouDiff)

        for box in boxes:
            cv.rectangle(frame, box, (0, 255, 0))
            cv.rectangle(frame, np.array(box), (0, 255, 0))
            cv.rectangle(frame, tuple(box), (0, 255, 0))
            cv.rectangle(frame, list(box), (0, 255, 0))


    def test_classification_model(self):
        img_path = self.find_dnn_file("dnn/googlenet_0.png")
        weights = self.find_dnn_file("dnn/squeezenet_v1.1.onnx", required=False)
        ref = np.load(self.find_dnn_file("dnn/squeezenet_v1.1_prob.npy"))
        if weights is None:
            raise unittest.SkipTest("Missing DNN test files (dnn/squeezenet_v1.1.onnx). Verify OPENCV_DNN_TEST_DATA_PATH configuration parameter.")

        frame = cv.imread(img_path)
        model = cv.dnn_ClassificationModel(weights)
        model.setInputSize(227, 227)
        model.setInputCrop(True)

        out = model.predict(frame)
        normAssert(self, out, ref)


    def test_textdetection_model(self):
        img_path = self.find_dnn_file("dnn/text_det_test1.png")
        weights = self.find_dnn_file("dnn/onnx/models/DB_TD500_resnet50.onnx", required=False)
        if weights is None:
            raise unittest.SkipTest("Missing DNN test files (onnx/models/DB_TD500_resnet50.onnx). Verify OPENCV_DNN_TEST_DATA_PATH configuration parameter.")

        frame = cv.imread(img_path)
        scale = 1.0 / 255.0
        size = (736, 736)
        mean = (122.67891434, 116.66876762, 104.00698793)

        model = cv.dnn_TextDetectionModel_DB(weights)
        model.setInputParams(scale, size, mean)
        out, _ = model.detect(frame)

        self.assertTrue(type(out) == tuple, msg='actual type {}'.format(str(type(out))))
        self.assertTrue(np.array(out).shape == (2, 4, 2))


    def test_face_detection(self):
        model = self.find_dnn_file('dnn/onnx/models/yunet-202605.onnx', required=False)
        img = self.get_sample('gpu/lbpcascade/er.png')

        ref = [[1, 339.62445, 35.32416, 30.754604, 40.202126, 0.9302596],
               [1, 140.63962, 255.55545, 32.832615, 41.767395, 0.916015],
               [1, 68.39314, 126.74046, 30.29324, 39.14823, 0.90639645],
               [1, 119.57139, 48.482178, 30.600697, 40.485996, 0.906021],
               [1, 259.0921, 229.30713, 31.088186, 39.74022, 0.90490955],
               [1, 405.69778, 87.28158, 33.393406, 42.96226, 0.8996978]]

        print('\n')
        for backend, target in self.dnnBackendsAndTargets:
            printParams(backend, target)

            net = cv.FaceDetectorYN.create(
                model=model,
                config="",
                input_size=img.shape[:2],
                score_threshold=0.3,
                nms_threshold=0.45,
                top_k=5000,
                backend_id=backend,
                target_id=target
            )

            out = net.detect(img)
            out = out[1]
            out = out.reshape(-1, 15)

            ref = np.array(ref, np.float32)
            refClassIds, testClassIds = ref[:, 0], np.ones(out.shape[0], np.float32)
            refScores, testScores = ref[:, -1], out[:, -1]
            refBoxes, testBoxes = ref[:, 1:5], out[:, 0:4]

            normAssertDetections(self, refClassIds, refScores, refBoxes, testClassIds,
                                 testScores, testBoxes, 0.5)

    def test_nms(self):
        confs = (1, 1)
        rects = ((0, 0, 0.4, 0.4), (0, 0, 0.2, 0.4)) # 0.5 overlap

        self.assertTrue(all(cv.dnn.NMSBoxes(rects, confs, 0, 0.6).ravel() == (0, 1)))

    # BUG: https://github.com/opencv/opencv/issues/26200
    @unittest.skip("custom layers are partially broken with transition to the new dnn engine")
    def test_custom_layer(self):
        class CropLayer(object):
            def __init__(self, params, blobs):
                self.xstart = 0
                self.xend = 0
                self.ystart = 0
                self.yend = 0
            # Our layer receives two inputs. We need to crop the first input blob
            # to match a shape of the second one (keeping batch size and number of channels)
            def getMemoryShapes(self, inputs):
                inputShape, targetShape = inputs[0], inputs[1]
                batchSize, numChannels = inputShape[0], inputShape[1]
                height, width = targetShape[2], targetShape[3]
                self.ystart = (inputShape[2] - targetShape[2]) // 2
                self.xstart = (inputShape[3] - targetShape[3]) // 2
                self.yend = self.ystart + height
                self.xend = self.xstart + width
                return [[batchSize, numChannels, height, width]]
            def forward(self, inputs):
                return [inputs[0][:,:,self.ystart:self.yend,self.xstart:self.xend]]

        cv.dnn_registerLayer('CropCaffe', CropLayer)

        # Skipped: Requires ONNX custom layer multi-input support and Python binding fixes for Net.connect (see #26200).
        cv.dnn_unregisterLayer('CropCaffe')

    # check that dnn module can work with 3D tensor as input for network
    def test_input_3d(self):
        model = self.find_dnn_file('dnn/onnx/models/hidden_lstm.onnx')
        input_file = self.find_dnn_file('dnn/onnx/data/input_hidden_lstm.npy')
        output_file = self.find_dnn_file('dnn/onnx/data/output_hidden_lstm.npy')
        if model is None:
            raise unittest.SkipTest("Missing DNN test files (dnn/onnx/models/hidden_lstm.onnx). "
                                    "Verify OPENCV_DNN_TEST_DATA_PATH configuration parameter.")
        if input_file is None or output_file is None:
            raise unittest.SkipTest("Missing DNN test files (dnn/onnx/data/{input/output}_hidden_lstm.npy). "
                                    "Verify OPENCV_DNN_TEST_DATA_PATH configuration parameter.")

        input = np.load(input_file)
        gold_output = np.load(output_file)

        for backend, target in self.dnnBackendsAndTargets:
            printParams(backend, target)

            net = cv.dnn.readNet(model, engine=cv.dnn.ENGINE_CLASSIC)

            net.setPreferableBackend(backend)
            net.setPreferableTarget(target)

            # Check whether 3d shape is parsed correctly for setInput
            net.setInput(input)

            # Case 0: test API `forward(const String& outputName = String()`
            real_output = net.forward() # Retval is a np.array of shape [2, 5, 3]
            normAssert(self, real_output, gold_output, "Case 1", getDefaultThreshold(target))

            '''
            Pre-allocate output memory with correct shape.
            Normally Python users do not use in this way,
            but we have to test it since we design API in this way
            '''
            # Case 1: a np.array with a string of output name.
            #         It tests API `forward(OutputArrayOfArrays outputBlobs, const String& outputName = String()`
            #         when outputBlobs is a np.array and we expect it to be the only output.
            real_output = np.empty([2, 5, 3], dtype=np.float32)
            real_output = net.forward(real_output, "237") # Retval is a tuple with a np.array of shape [2, 5, 3]
            normAssert(self, real_output, gold_output, "Case 1", getDefaultThreshold(target))

            # Case 2: a tuple of np.array with a string of output name.
            #         It tests API `forward(OutputArrayOfArrays outputBlobs, const String& outputName = String()`
            #         when outputBlobs is a container of several np.array and we expect to save all outputs accordingly.
            real_output = tuple(np.empty([2, 5, 3], dtype=np.float32))
            real_output = net.forward(real_output, "237") # Retval is a tuple with a np.array of shape [2, 5, 3]
            normAssert(self, real_output, gold_output, "Case 2", getDefaultThreshold(target))

            # Case 3: a tuple of np.array with a string of output name.
            #         It tests API `forward(OutputArrayOfArrays outputBlobs, const std::vector<String>& outBlobNames)`
            real_output = tuple(np.empty([2, 5, 3], dtype=np.float32))
            # Note that it does not support parsing a list , e.g. ["237"]
            real_output = net.forward(real_output, ("237")) # Retval is a tuple with a np.array of shape [2, 5, 3]
            normAssert(self, real_output, gold_output, "Case 3", getDefaultThreshold(target))

    def test_set_param_3d(self):
        model_path = self.find_dnn_file('dnn/onnx/models/matmul_3d_init.onnx')
        input_file = self.find_dnn_file('dnn/onnx/data/input_matmul_3d_init.npy')
        output_file = self.find_dnn_file('dnn/onnx/data/output_matmul_3d_init.npy')

        input = np.load(input_file)
        output = np.load(output_file)

        for backend, target in self.dnnBackendsAndTargets:
            printParams(backend, target)

            net = cv.dnn.readNet(model_path, "", "", engine=cv.dnn.ENGINE_CLASSIC)

            node_name = net.getLayerNames()[0]
            w = net.getParam(node_name, 0) # returns the original tensor of three-dimensional shape
            net.setParam(node_name, 0, w)  # set param once again to see whether tensor is converted with correct shape

            net.setPreferableBackend(backend)
            net.setPreferableTarget(target)

            net.setInput(input)
            res_output = net.forward()

            normAssert(self, output, res_output, "", getDefaultThreshold(target))

    def test_scalefactor_assign(self):
        params = cv.dnn.Image2BlobParams()
        self.assertEqual(params.scalefactor, (1.0, 1.0, 1.0, 1.0))
        params.scalefactor = 2.0
        self.assertEqual(params.scalefactor, (2.0, 0.0, 0.0, 0.0))

    def test_net_builder(self):
        net = cv.dnn.Net()
        params = {
            "kernel_w": 3,
            "kernel_h": 3,
            "stride_w": 3,
            "stride_h": 3,
            "pool": "max",
        }
        net.addLayerToPrev("pool", "Pooling", cv.CV_32F, params)

        inp = np.random.standard_normal([1, 2, 9, 12]).astype(np.float32)
        net.setInput(inp)
        out = net.forward()
        self.assertEqual(out.shape, (1, 2, 3, 4))

    def test_bool_operator(self):
        n = self.find_dnn_file('dnn/onnx/models/and_op.onnx')

        x = np.random.randint(0, 2, [5], dtype=np.bool_)
        y = np.random.randint(0, 2, [5], dtype=np.bool_)
        o = x & y

        net = cv.dnn.readNet(n)

        names = ["x", "y"]
        net.setInputsNames(names)
        net.setInput(x, names[0])
        net.setInput(y, names[1])

        out = net.forward()

        self.assertTrue(np.all(out == o))


if __name__ == '__main__':
    NewOpenCVTests.bootstrap()



#!/usr/bin/env python
'''
This file is part of OpenCV project.
It is subject to the license terms in the LICENSE file found in the top-level directory
of this distribution and at http://opencv.org/license.html.

This sample deblurs the given blurry image.

Copyright (C) 2025, Bigvision LLC.

How to use:
    Sample command to run:
        `python deblurring.py`

    You can download NAFNet deblurring model using
        `python download_models.py NAFNet`

    References:
      Github: https://github.com/megvii-research/NAFNet
      PyTorch model: https://drive.google.com/file/d/14D4V4raNYIOhETfcuuLI3bGLB-OYIv6X/view

      PyTorch model was converted to ONNX and then ONNX model was further quantized using block quantization from [opencv_zoo](https://github.com/opencv/opencv_zoo/blob/main/tools/quantize/block_quantize.py)

    Set environment variable OPENCV_DOWNLOAD_CACHE_DIR to point to the directory where models are downloaded. Also, point OPENCV_SAMPLES_DATA_PATH to opencv/samples/data.
'''

import argparse
import cv2 as cv
import numpy as np
from common import *

def help():
    print(
        '''
        Use this script for image deblurring using OpenCV.

        Firstly, download required models i.e. NAFNet using `download_models.py` (if not already done). Set environment variable OPENCV_DOWNLOAD_CACHE_DIR to specify where models should be downloaded. Also, point OPENCV_SAMPLES_DATA_PATH to opencv/samples/data.

        To run:
        Example: python deblurring.py [--input=<image_name>]

        Deblurring model path can also be specified using --model argument.
        '''
    )

def get_args_parser():
    backends = ("default", "openvino", "opencv", "vkcom", "cuda")
    targets = ("cpu", "opencl", "opencl_fp16", "ncs2_vpu", "hddl_vpu", "vulkan", "cuda", "cuda_fp16")

    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--zoo', default=os.path.join(os.path.dirname(os.path.abspath(__file__)), 'models.yml'),
                        help='An optional path to file with preprocessing parameters.')
    parser.add_argument('--input', '-i', default="licenseplate_motion.jpg", help='Path to image file.', required=False)
    parser.add_argument('--backend', default="default", type=str, choices=backends,
            help="Choose one of computation backends: "
            "default: automatically (by default), "
            "openvino: Intel's Deep Learning Inference Engine (https://software.intel.com/openvino-toolkit), "
            "opencv: OpenCV implementation, "
            "vkcom: VKCOM, "
            "cuda: CUDA, "
            "webnn: WebNN")
    parser.add_argument('--target', default="cpu", type=str, choices=targets,
            help="Choose one of target computation devices: "
            "cpu: CPU target (by default), "
            "opencl: OpenCL, "
            "opencl_fp16: OpenCL fp16 (half-float precision), "
            "ncs2_vpu: NCS2 VPU, "
            "hddl_vpu: HDDL VPU, "
            "vulkan: Vulkan, "
            "cuda: CUDA, "
            "cuda_fp16: CUDA fp16 (half-float preprocess)")
    args, _ = parser.parse_known_args()
    add_preproc_args(args.zoo, parser, 'deblurring', prefix="", alias="NAFNet")
    parser = argparse.ArgumentParser(parents=[parser],
                                        description='Image deblurring using OpenCV.',
                                        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    return parser.parse_args()

def main():
    if hasattr(args, 'help'):
        help()
        exit(1)

    args.model = findModel(args.model, args.sha1)

    engine = cv.dnn.ENGINE_AUTO

    if args.backend != "default" or args.target != "cpu":
        engine = cv.dnn.ENGINE_CLASSIC

    net = cv.dnn.readNetFromONNX(args.model, engine)
    net.setPreferableBackend(get_backend_id(args.backend))
    net.setPreferableTarget(get_target_id(args.target))

    input_image = cv.imread(findFile(args.input))
    image = input_image.copy()
    height, width = image.shape[:2]

    image_blob = cv.dnn.blobFromImage(image, args.scale, (width, height), args.mean, args.rgb, False)
    net.setInput(image_blob)
    out = net.forward()

    # Postprocessing
    output = out[0]
    output = np.transpose(output, (1, 2, 0))
    output = np.clip(output * 255.0, 0, 255).astype(np.uint8)
    out_image = cv.cvtColor(output, cv.COLOR_RGB2BGR)

    cv.imshow("input image: ", input_image)
    cv.imshow("output image: ", out_image)
    cv.waitKey(0)

if __name__ == '__main__':
    args = get_args_parser()
    main()



'''
This is a sample script to run GPT-2 inference in OpenCV using ONNX model.
The script loads the GPT-2 model and runs inference on a given prompt.
Currently script only works with fixed size window, that means
you will have to specify prompt of the same length as when model was exported to ONNX.


Exporting GPT-2 model to ONNX.
To export GPT-2 model to ONNX, you can use the following procedure:

1. Clone fork of Andrej Karpathy's GPT-2 repository:

    git clone -b fix-dynamic-axis-export  https://github.com/nklskyoy/build-nanogpt

2. Install the required dependencies:

    pip install -r requirements.txt

3  Export the model to ONNX:

    python export2onnx.py --promt=<Any-promt-you-want>


Run the script:
1. Install the required dependencies:

    pip install tiktoken==0.7.0 numpy tqdm

2. Run the script:
    python gpt2_inference.py --model=<path-to-onnx-model> --tokenizer_path=<path-to-tokenizer-config> --prompt=<use-promt-of-the-same-length-used-while-exporting>
'''

import numpy as np
import argparse
import cv2 as cv

def parse_args():
    parser = argparse.ArgumentParser(description='Use this script to run GPT-2 inference in OpenCV',
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--model', type=str, required=True, help='Path to GPT-2 model ONNX model file.')
    parser.add_argument('--tokenizer_path', type=str, required=True, help='Path to GPT-2 tokenizer config file.')
    parser.add_argument("--prompt", type=str, default="Hello, I'm a language model,", help="Prompt to start with.")
    parser.add_argument("--max_seq_len", type=int, default=32, help="Number of tokens to continue.")
    parser.add_argument("--seed", type=int, default=0, help="Random seed")
    return parser.parse_args()

def stable_softmax(logits):
    exp_logits = np.exp(logits - np.max(logits, axis=-1, keepdims=True))
    return exp_logits / np.sum(exp_logits, axis=-1, keepdims=True)



def gpt2_inference(net, prompt, max_length, tokenizer):

    print("Inferencing GPT-2 model...")

    tokens = tokenizer.encode(prompt).reshape(1,-1)

    stop_tokens = (50256, ) ## could be extended to include more stop tokens
    while 0 < max_length and tokens[:, -1] not in stop_tokens:

        net.setInputsNames(['idx'])
        net.setInput(tokens, 'idx')
        logits = net.forward()
        logits = logits[:, -1, :]  # (B, vocab_size)

        # use hard sampling
        new_idx = np.argmax(logits.reshape(-1)).reshape(1,1)

        tokens = np.concatenate((tokens, new_idx), axis=1)

        max_length -= 1
    return tokens



if __name__ == '__main__':

    args = parse_args()
    print("Preparing GPT-2 model...")
    max_length = args.max_seq_len
    prompt = args.prompt
    tokenizer_path = args.tokenizer_path

    net = cv.dnn.readNetFromONNX(args.model, cv.dnn.ENGINE_NEW)
    tokenizer = cv.dnn.Tokenizer.load(tokenizer_path)

    tokens = gpt2_inference(net, prompt, max_length, tokenizer)
    print(tokenizer.decode(tokens[0]))



import numpy as np
import cv2 as cv

from tests_common import NewOpenCVTests

class volume_test(NewOpenCVTests):
    def test_VolumeDefault(self):
        depthPath = 'cv/rgbd/depth.png'
        depth = self.get_sample(depthPath, cv.IMREAD_ANYDEPTH).astype(np.float32)
        if depth.size <= 0:
            raise Exception('Failed to load depth file: %s' % depthPath)

        Rt = np.eye(4)
        volume = cv.Volume()
        volume.integrate(depth, Rt)

        size = (480, 640, 4)
        points  = np.zeros(size, np.float32)
        normals = np.zeros(size, np.float32)

        Kraycast = np.array([[525. ,   0. , 319.5],
                             [  0. , 525. , 239.5],
                             [  0. ,   0. ,   1. ]])

        volume.raycastEx(Rt, size[0], size[1], Kraycast, points, normals)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

        volume.raycast(Rt, points, normals)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

    def test_VolumeTSDF(self):
        depthPath = 'cv/rgbd/depth.png'
        depth = self.get_sample(depthPath, cv.IMREAD_ANYDEPTH).astype(np.float32)
        if depth.size <= 0:
            raise Exception('Failed to load depth file: %s' % depthPath)

        Rt = np.eye(4)

        settings = cv.VolumeSettings(cv.VolumeType_TSDF)
        volume = cv.Volume(cv.VolumeType_TSDF, settings)
        volume.integrate(depth, Rt)

        size = (480, 640, 4)
        points  = np.zeros(size, np.float32)
        normals = np.zeros(size, np.float32)

        Kraycast = settings.getCameraRaycastIntrinsics()
        volume.raycastEx(Rt, size[0], size[1], Kraycast, points, normals)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

        volume.raycast(Rt, points, normals)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

    def test_VolumeHashTSDF(self):
        depthPath = 'cv/rgbd/depth.png'
        depth = self.get_sample(depthPath, cv.IMREAD_ANYDEPTH).astype(np.float32)
        if depth.size <= 0:
            raise Exception('Failed to load depth file: %s' % depthPath)

        Rt = np.eye(4)
        settings = cv.VolumeSettings(cv.VolumeType_HashTSDF)
        volume = cv.Volume(cv.VolumeType_HashTSDF, settings)
        volume.integrate(depth, Rt)

        size = (480, 640, 4)
        points  = np.zeros(size, np.float32)
        normals = np.zeros(size, np.float32)

        Kraycast = settings.getCameraRaycastIntrinsics()
        volume.raycastEx(Rt, size[0], size[1], Kraycast, points, normals)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

        volume.raycast(Rt, points, normals)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

    def test_VolumeColorTSDF(self):
        depthPath = 'cv/rgbd/depth.png'
        rgbPath = 'cv/rgbd/rgb.png'
        depth = self.get_sample(depthPath, cv.IMREAD_ANYDEPTH).astype(np.float32)
        rgb = self.get_sample(rgbPath, cv.IMREAD_ANYCOLOR).astype(np.float32)

        if depth.size <= 0:
            raise Exception('Failed to load depth file: %s' % depthPath)
        if rgb.size <= 0:
            raise Exception('Failed to load RGB file: %s' % rgbPath)

        Rt = np.eye(4)
        settings = cv.VolumeSettings(cv.VolumeType_ColorTSDF)
        volume = cv.Volume(cv.VolumeType_ColorTSDF, settings)
        volume.integrateColor(depth, rgb, Rt)

        size = (480, 640, 4)
        points  = np.zeros(size, np.float32)
        normals = np.zeros(size, np.float32)
        colors = np.zeros(size, np.float32)

        Kraycast = settings.getCameraRaycastIntrinsics()
        volume.raycastExColor(Rt, size[0], size[1], Kraycast, points, normals, colors)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)

        volume.raycastColor(Rt, points, normals, colors)

        self.assertEqual(points.shape, size)
        self.assertEqual(points.shape, normals.shape)
        self.assertEqual(points.shape, colors.shape)


if __name__ == '__main__':
    NewOpenCVTests.bootstrap()



#!/usr/bin/env python
"""Algorithm serialization test."""
import tempfile
import os
import cv2 as cv
from tests_common import NewOpenCVTests


class algorithm_rw_test(NewOpenCVTests):
    def test_algorithm_rw(self):
        fd, fname = tempfile.mkstemp(prefix="opencv_python_algorithm_", suffix=".yml")
        os.close(fd)

        # some arbitrary non-default parameters
        gold = cv.ORB_create(nfeatures=200, scaleFactor=1.3, nlevels=5, edgeThreshold=28)
        gold.write(cv.FileStorage(fname, cv.FILE_STORAGE_WRITE), "ORB")

        fs = cv.FileStorage(fname, cv.FILE_STORAGE_READ)
        algorithm = cv.ORB_create()
        algorithm.read(fs.getNode("ORB"))

        self.assertEqual(algorithm.getMaxFeatures(), 200)
        self.assertAlmostEqual(algorithm.getScaleFactor(), 1.3, places=6)
        self.assertEqual(algorithm.getNLevels(), 5)
        self.assertEqual(algorithm.getEdgeThreshold(), 28)

        os.remove(fname)



'''
This is a sample script to run Qwen2.5 inference in OpenCV using ONNX model.
The script loads the Qwen2.5 model and runs inference on a given prompt using
the ChatML format (<|im_start|> / <|im_end|> special tokens).

Model: https://huggingface.co/Qwen/Qwen2.5-0.5B-Instruct

Exporting Qwen2.5 model to ONNX:

1. Install the required dependencies:

    pip install optimum[exporters] optimum-onnx[onnxruntime] torch transformers

2. Export the model to ONNX:

    Without KV-cache:

        optimum-cli export onnx --model Qwen/Qwen2.5-0.5B-Instruct --task causal-lm qwen2.5_instruct_onnx/

    With KV-cache (recommended, faster autoregressive inference):

        optimum-cli export onnx --model Qwen/Qwen2.5-0.5B-Instruct --task causal-lm-with-past qwen2.5_instruct_onnx_with_past/


Run the script:
1. Install the required dependencies:

    pip install numpy

2. Run the script:

    Without KV-cache (causal-lm export):

        python qwen_inference.py --model=<path-to-onnx-model> \
                                 --tokenizer_path=<path-to-qwen2.5-config.json> \
                                 --prompt="What is OpenCV?"

    With KV-cache (causal-lm-with-past export):

        python qwen_inference.py --model=<path-to-onnx-model> \
                                 --tokenizer_path=<path-to-qwen2.5-config.json> \
                                 --prompt="What is OpenCV?" \
                                 --use_kv_cache
'''

import numpy as np
import argparse
import cv2 as cv

def parse_args():
    parser = argparse.ArgumentParser(description='Use this script to run Qwen2.5 inference in OpenCV',
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--model', type=str, required=True, help='Path to Qwen2.5 ONNX model file.')
    parser.add_argument('--tokenizer_path', type=str, required=True, help='Path to Qwen2.5 tokenizer config.json.')
    parser.add_argument('--prompt', type=str, default='What is OpenCV?', help='User prompt.')
    parser.add_argument('--max_new_tokens', type=int, default=64, help='Maximum number of new tokens to generate.')
    parser.add_argument('--use_kv_cache', action='store_true', default=False, help='Enable KV-cache for faster inference (requires causal-lm-with-past export).')
    parser.add_argument('--seed', type=int, default=0, help='Random seed.')
    return parser.parse_args()

def build_chatml_prompt(user_prompt):
    '''Wrap user prompt in Qwen2.5 ChatML format.'''
    return '<|im_start|>user\n' + user_prompt + '<|im_end|>\n<|im_start|>assistant\n'

def qwen_inference(net, prompt, max_new_tokens, tokenizer, use_kv_cache=True):

    print("Inferencing Qwen2.5 model...")

    tokens = list(tokenizer.encode(prompt))
    input_ids = np.array(tokens, dtype=np.int64).reshape(1, -1)

    # Qwen2.5 special token IDs
    im_end_id = 151645   # <|im_end|>
    eos_id    = 151643   # <|endoftext|>
    stop_ids  = (im_end_id, eos_id)

    generated = []

    if use_kv_cache:
        net.enableKVCache()
        prompt_len = input_ids.shape[1]

        # Prefill: process full prompt once to populate KV-cache
        net.setInput(input_ids, 'input_ids')
        net.setInput(np.ones((1, prompt_len), dtype=np.int64), 'attention_mask')
        net.setInput(np.arange(prompt_len, dtype=np.int64).reshape(1, -1), 'position_ids')
        logits = net.forward()
        new_id = int(np.argmax(logits[:, -1, :].reshape(-1)))
        generated = [new_id]

        # Generate: feed one new token per step; OpenCV routes present.* -> past_key_values.*
        for _ in range(max_new_tokens - 1):
            if new_id in stop_ids:
                break
            cur_len = prompt_len + len(generated)
            net.setInput(np.array([[new_id]], dtype=np.int64), 'input_ids')
            net.setInput(np.ones((1, cur_len), dtype=np.int64), 'attention_mask')
            net.setInput(np.array([[cur_len - 1]], dtype=np.int64), 'position_ids')
            logits = net.forward()
            new_id = int(np.argmax(logits[:, -1, :].reshape(-1)))
            generated.append(new_id)
    else:
        # Without KV-cache: feed full growing sequence each step
        for _ in range(max_new_tokens):
            seq_len = input_ids.shape[1]
            net.setInput(input_ids, 'input_ids')
            net.setInput(np.ones((1, seq_len), dtype=np.int64), 'attention_mask')
            net.setInput(np.arange(seq_len, dtype=np.int64).reshape(1, -1), 'position_ids')
            logits = net.forward()
            new_id = int(np.argmax(logits[:, -1, :].reshape(-1)))
            if new_id in stop_ids:
                break
            generated.append(new_id)
            input_ids = np.concatenate([input_ids, [[new_id]]], axis=1)

    return np.array([tokens + generated], dtype=np.int64)

if __name__ == '__main__':

    args = parse_args()
    np.random.seed(args.seed)

    print("Preparing Qwen2.5 model...")
    tokenizer = cv.dnn.Tokenizer.load(args.tokenizer_path)

    net = cv.dnn.readNetFromONNX(args.model, cv.dnn.ENGINE_NEW)

    chatml_prompt = build_chatml_prompt(args.prompt)
    print(f"Prompt:\n{chatml_prompt}")

    prompt_len = len(tokenizer.encode(chatml_prompt))
    tokens = qwen_inference(net, chatml_prompt, args.max_new_tokens, tokenizer, args.use_kv_cache)
    response = tokenizer.decode(tokens[0][prompt_len:].tolist())
    print(f"Response:\n{response}")



import cv2 as cv
import numpy as np
import argparse
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from dnn.common import *

def get_args_parser(func_args):
    backends = ("default", "openvino", "opencv", "vkcom", "cuda")
    targets = ("cpu", "opencl", "opencl_fp16", "ncs2_vpu", "hddl_vpu", "vulkan", "cuda", "cuda_fp16")

    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--zoo', default=os.path.join(os.path.dirname(os.path.abspath(__file__)), '../dnn', 'models.yml'),
                        help='An optional path to file with preprocessing parameters.')
    parser.add_argument('--input', help='Path to input image or video file. Skip this argument to capture frames from a camera.', default=0, required=False)
    parser.add_argument('--method', help='choose method: dexined or canny', default='canny', required=False)
    parser.add_argument('--chart_type', type=int, default=0,
                        help='chartType: 0-Standard, 1-DigitalSG, 2-Vinyl, default:0')
    parser.add_argument('--num_charts', type=int, default=1,
                        help='Maximum number of charts in the image')
    parser.add_argument('--backend', default="default", type=str, choices=backends,
                    help="Choose one of computation backends: "
                         "default: automatically (by default), "
                         "openvino: Intel's Deep Learning Inference Engine (https://software.intel.com/openvino-toolkit), "
                         "opencv: OpenCV implementation, "
                         "vkcom: VKCOM, "
                         "cuda: CUDA, "
                         "webnn: WebNN")
    parser.add_argument('--target', default="cpu", type=str, choices=targets,
                    help="Choose one of target computation devices: "
                         "cpu: CPU target (by default), "
                         "opencl: OpenCL, "
                         "opencl_fp16: OpenCL fp16 (half-float precision), "
                         "ncs2_vpu: NCS2 VPU, "
                         "hddl_vpu: HDDL VPU, "
                         "vulkan: Vulkan, "
                         "cuda: CUDA, "
                         "cuda_fp16: CUDA fp16 (half-float preprocess)")

    args, _ = parser.parse_known_args()
    add_preproc_args(args.zoo, parser, 'mcc', 'mcc')
    parser = argparse.ArgumentParser(parents=[parser],
                                     description='''
        To run:
            Default:
                python macbeth_chart_detection.py --input=path/to/your/input/image/or/video (don't give --input flag if want to use device camera)
            DNN model:
                python macbeth_chart_detection.py mcc --input=path/to/your/input/image/or/video

        Model path can also be specified using --model argument. And config path can be specified using --config.
        ''', formatter_class=argparse.RawTextHelpFormatter)
    return parser.parse_args(func_args)

def process_frame(frame, detector, num_charts):
    image_copy = frame.copy()
    if not detector.process(frame, num_charts):
        return None, None

    checkers = detector.getListColorChecker()
    detector.draw(checkers, frame)

    src = checkers[0].getChartsRGB(False)
    tgt = detector.getRefColors()

    cv.imshow("image result | Press ESC to quit", frame)
    cv.imshow("original", image_copy)

    return src, tgt

def main(func_args=None):
    args = get_args_parser(func_args)

    if not (0 <= args.chart_type <= 2):
        raise ValueError("chartType must be 0, 1, or 2")

    if os.getenv('OPENCV_SAMPLES_DATA_PATH') is not None:
        try:
            args.model = findModel(args.model, args.sha1)
            args.config = findModel(args.config, args.config_sha1)
        except:
            print("[WARN] Model file not provided, using default detector. Pass model using --model and config using --config to use dnn based detector.\n\n")
            args.model = None
            args.config = None
    else:
        args.model = None
        args.config = None
        print("[WARN] Model file not provided, using default detector. Pass model using --model and config using --config to use dnn based detector. Or, set OPENCV_SAMPLES_DATA_PATH environment variable.\n\n")

    if args.model and args.config:
        # Load the DNN from TensorFlow model
        engine = cv.dnn.ENGINE_AUTO
        if args.backend != "default" or args.target != "cpu":
            engine = cv.dnn.ENGINE_CLASSIC
        net = cv.dnn.readNetFromTensorflow(args.model, args.config, engine)
        net.setPreferableBackend(get_backend_id(args.backend))
        net.setPreferableTarget(get_target_id(args.target))

        detector = cv.mcc_CCheckerDetector.create(net)
        print("Detecting checkers using neural network.")
    else:
        detector = cv.mcc_CCheckerDetector.create()
        print("Detecting checkers using default method (no DNN).")

    detector.setColorChartType(args.chart_type)
    is_video = True

    if args.input:
        image = cv.imread(findFile(args.input))
        if image is not None:
            is_video = False
        else:
            cap = cv.VideoCapture(findFile(args.input))
    else:
        cap = cv.VideoCapture(0)

    if is_video:
        print("To print the actual colors and reference colors for current frame press SPACEBAR. To resume press SPACEBAR again")

        while True:
            ret, frame = cap.read()
            if not ret:
                break

            src, tgt = process_frame(frame, detector, args.num_charts)

            key = cv.waitKey(10) & 0xFF
            if key == ord(' '):
                if src is None or tgt is None:
                    print("No color chart detected!!")
                else:
                    print("Actual colors: ", src)
                    print("Reference colors: ", tgt)
                    print("Press spacebar to resume.")

                    cv.waitKey(0)
                    print("Resumed! Processing continues...")
            elif key == 27:
                exit(0)
        if src is not None or tgt is not None:
            print("Actual colors: ", src)
            print("Reference colors: ", tgt)
    else:
        src, tgt = process_frame(image, detector, args.num_charts)
        if src is None or tgt is None:
            print("No color chart detected!!")
        else:
            print("Actual colors: ", src)
            print("Reference colors: ", tgt)
            cv.waitKey(0)

    cv.destroyAllWindows()

if __name__ == "__main__":
    main()



import sys
import os
import cv2 as cv


def add_argument(zoo, parser, name, help, required=False, default=None, type=None, action=None, nargs=None, alias=None):
    if alias is not None:
        modelName = alias
    elif len(sys.argv) > 1:
        modelName = sys.argv[1]
    else:
        return

    if os.path.isfile(zoo):
        fs = cv.FileStorage(zoo, cv.FILE_STORAGE_READ)
        node = fs.getNode(modelName)
        if not node.empty():
            value = node.getNode(name)
            if "sha1" in name:
                prefix = name.replace("sha1", "")
                value = node.getNode(prefix + "load_info")
                if prefix == "config_":
                    value = value.getNode("sha1")
                else:
                    value = value.getNode(name)
            if "download_sha" in name:
                prefix = name.replace("download_sha", "")
                value = node.getNode(prefix + "load_info")
                value = value.getNode(name)
            if not value.empty():
                if value.isReal():
                    default = value.real()
                elif value.isString():
                    default = value.string()
                elif value.isInt():
                    default = int(value.real())
                elif value.isSeq():
                    default = []
                    for i in range(value.size()):
                        v = value.at(i)
                        if v.isInt():
                            default.append(int(v.real()))
                        elif v.isReal():
                            default.append(v.real())
                        else:
                            print('Unexpected value format')
                            exit(0)
                else:
                    print('Unexpected field format')
                    exit(0)
                required = False

    if action == 'store_true':
        default = 1 if default == 'true' else (0 if default == 'false' else default)
        assert(default is None or default == 0 or default == 1)
        parser.add_argument('--' + name, required=required, help=help, default=bool(default),
                            action=action)
    else:
        parser.add_argument('--' + name, required=required, help=help, default=default,
                            action=action, nargs=nargs, type=type)


def add_preproc_args(zoo, parser, sample, alias=None, prefix=""):
    aliases = []
    if os.path.isfile(zoo) and prefix == "":
        fs = cv.FileStorage(zoo, cv.FILE_STORAGE_READ)
        root = fs.root()
        for name in root.keys():
            model = root.getNode(name)
            if model.getNode('sample').string() == sample:
                aliases.append(name)
    if len(aliases):
        parser.add_argument(prefix+'alias', nargs='?', choices=aliases,
                            help='An alias name of model to extract preprocessing parameters from models.yml file.')

    add_argument(zoo, parser, prefix+'model',
                 help='Path to a binary file of model contains trained weights. '
                      'It could be a file with extensions .caffemodel (Caffe), '
                      '.pb (TensorFlow), .bin (OpenVINO)', alias=alias)
    add_argument(zoo, parser, prefix+'config',
                 help='Path to a text file of model contains network configuration. '
                      'It could be a file with extensions .prototxt (Caffe), .pbtxt or .config (TensorFlow), .xml (OpenVINO)', alias=alias)
    add_argument(zoo, parser, prefix+'mean', nargs='+', type=float, default=[0, 0, 0],
                 help='Preprocess input image by subtracting mean values. '
                      'Mean values should be in BGR order.', alias=alias)
    add_argument(zoo, parser, prefix+'std', nargs='+', type=float, default=[0, 0, 0],
                 help='Preprocess input image by dividing on a standard deviation.', alias=alias)
    add_argument(zoo, parser, prefix+'scale', type=float, default=1.0,
                 help='Preprocess input image by multiplying on a scale factor.', alias=alias)
    add_argument(zoo, parser, prefix+'width', type=int,
                 help='Preprocess input image by resizing to a specific width.', alias=alias)
    add_argument(zoo, parser, prefix+'height', type=int,
                 help='Preprocess input image by resizing to a specific height.', alias=alias)
    add_argument(zoo, parser, prefix+'rgb', action='store_true',
                 help='Indicate that model works with RGB input images instead BGR ones.', alias=alias)
    add_argument(zoo, parser, prefix+'labels',
                 help='Optional path to a text file with names of labels to label detected objects.', alias=alias)
    add_argument(zoo, parser, prefix+'postprocessing', type=str,
                 help='Post-processing kind depends on model topology.', alias=alias)
    add_argument(zoo, parser, prefix+'background_label_id', type=int, default=-1,
                 help='An index of background class in predictions. If not negative, exclude such class from list of classes.', alias=alias)
    add_argument(zoo, parser, prefix+'sha1', type=str,
                 help='Optional path to hashsum of downloaded model to be loaded from models.yml', alias=alias)
    add_argument(zoo, parser, prefix+'config_sha1', type=str,
                 help='Optional path to hashsum of downloaded config to be loaded from models.yml', alias=alias)
    add_argument(zoo, parser, prefix+'download_sha', type=str,
                 help='Optional path to hashsum of downloaded model to be loaded from models.yml', alias=alias)

def findModel(filename, sha1):
    if filename:
        if os.path.exists(filename):
            return filename

        fpath = cv.samples.findFile(filename, False)
        if fpath:
            return fpath

        if os.getenv('OPENCV_DOWNLOAD_CACHE_DIR') is None:
            print('[WARN] Please specify a path to model download directory in OPENCV_DOWNLOAD_CACHE_DIR environment variable.')
            return findFile(filename)

        if os.path.exists(os.path.join(os.environ['OPENCV_DOWNLOAD_CACHE_DIR'], sha1, filename)):
            return os.path.join(os.environ['OPENCV_DOWNLOAD_CACHE_DIR'], sha1, filename)

        if os.path.exists(os.path.join(os.environ['OPENCV_DOWNLOAD_CACHE_DIR'], filename)):
            return os.path.join(os.environ['OPENCV_DOWNLOAD_CACHE_DIR'], filename)

    raise FileNotFoundError('File ' + filename + ' not found! Please specify a path to '
            'model download directory in OPENCV_DOWNLOAD_CACHE_DIR '
            'environment variable or pass a full path to ' + filename)

def findFile(filename):
    if filename:
        if os.path.exists(filename):
            return filename

        fpath = cv.samples.findFile(filename, False)
        if fpath:
            return fpath

        if os.getenv('OPENCV_SAMPLES_DATA_PATH') is None:
            print('[WARN] Please specify a path to `/samples/data` in OPENCV_SAMPLES_DATA_PATH environment variable.')
            exit(0)

        if os.path.exists(os.path.join(os.environ['OPENCV_SAMPLES_DATA_PATH'], filename)):
            return os.path.join(os.environ['OPENCV_SAMPLES_DATA_PATH'], filename)

        for path in ['OPENCV_DNN_TEST_DATA_PATH', 'OPENCV_TEST_DATA_PATH', 'OPENCV_SAMPLES_DATA_PATH']:
            try:
                extraPath = os.environ[path]
                absPath = os.path.join(extraPath, 'dnn', filename)
                if os.path.exists(absPath):
                    return absPath
            except KeyError:
                pass

    raise FileNotFoundError(
        'File ' + filename + ' not found! Please specify the path to '
        '/opencv/samples/data in the OPENCV_SAMPLES_DATA_PATH environment variable, '
        'or specify the path to opencv_extra/testdata in the OPENCV_DNN_TEST_DATA_PATH environment variable, '
        'or specify the path to the model download cache directory in the OPENCV_DOWNLOAD_CACHE_DIR environment variable, '
        'or pass the full path to ' + filename + '.'
    )


def get_backend_id(backend_name):
    backend_ids = {
        "default": cv.dnn.DNN_BACKEND_DEFAULT,
        "openvino": cv.dnn.DNN_BACKEND_INFERENCE_ENGINE,
        "opencv": cv.dnn.DNN_BACKEND_OPENCV,
        "vkcom": cv.dnn.DNN_BACKEND_VKCOM,
        "cuda": cv.dnn.DNN_BACKEND_CUDA
    }

    if backend_name not in backend_ids:
        raise ValueError(f"Invalid backend name: {backend_name}")

    return backend_ids[backend_name]

def get_target_id(target_name):
    target_ids = {
        "cpu": cv.dnn.DNN_TARGET_CPU,
        "opencl": cv.dnn.DNN_TARGET_OPENCL,
        "opencl_fp16": cv.dnn.DNN_TARGET_OPENCL_FP16,
        "ncs2_vpu": cv.dnn.DNN_TARGET_MYRIAD,
        "hddl_vpu": cv.dnn.DNN_TARGET_HDDL,
        "vulkan": cv.dnn.DNN_TARGET_VULKAN,
        "cuda": cv.dnn.DNN_TARGET_CUDA,
        "cuda_fp16": cv.dnn.DNN_TARGET_CUDA_FP16
    }
    if target_name not in target_ids:
        raise ValueError(f"Invalid target name: {target_name}")

    return target_ids[target_name]